@@ -1,23 +1,23 @@
 /*
- * //******************************************************************
- * //
- * // Copyright 2015 Intel Corporation.
- * //
- * //-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
- * //
- * // Licensed under the Apache License, Version 2.0 (the "License");
- * // you may not use this file except in compliance with the License.
- * // You may obtain a copy of the License at
- * //
- * //      http://www.apache.org/licenses/LICENSE-2.0
- * //
- * // Unless required by applicable law or agreed to in writing, software
- * // distributed under the License is distributed on an "AS IS" BASIS,
- * // WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * // See the License for the specific language governing permissions and
- * // limitations under the License.
- * //
- * //-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+ *******************************************************************
+ *
+ * Copyright 2015 Intel Corporation.
+ *
+ *-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ *
+ *-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  */
 
 package org.iotivity.base;
@@ -309,6 +309,18 @@ public class OcRepresentationTest extends InstrumentationTestCase {
         rep.setValue(representationK, representationArrVEmpty);
         OcRepresentation[] representationArrVEmptyA = rep.getValue(representationK);
         assertEquals(representationArrVEmpty.length, representationArrVEmptyA.length);
+
+        //byte
+        String byteK = "byteK";
+        byte[] byteArrV = {1, 2, 3, 4, 0, 5, 6, 7};
+        rep.setValue(byteK, byteArrV);
+        byte[] byteArrVa = rep.getValue(byteK);
+        assertTrue(Arrays.equals(byteArrV, byteArrVa));
+
+        byte[] byteArrVEmpty = {};
+        rep.setValue(byteK, byteArrVEmpty);
+        byte[] byteArrVEmptyA = rep.getValue(byteK);
+        assertTrue(Arrays.equals(byteArrVEmpty, byteArrVEmptyA));
     }
 
     public void testAttributeAccessBy2DType() throws OcException {
@@ -580,4 +592,4 @@ public class OcRepresentationTest extends InstrumentationTestCase {
             }
         }
     }
-}
\ No newline at end of file
+}
