@@ -1,23 +1,23 @@
 /*
- * //******************************************************************
- * //
- * // Copyright 2015 Intel Corporation.
- * //
- * //-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
- * //
- * // Licensed under the Apache License, Version 2.0 (the "License");
- * // you may not use this file except in compliance with the License.
- * // You may obtain a copy of the License at
- * //
- * //      http://www.apache.org/licenses/LICENSE-2.0
- * //
- * // Unless required by applicable law or agreed to in writing, software
- * // distributed under the License is distributed on an "AS IS" BASIS,
- * // WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
- * // See the License for the specific language governing permissions and
- * // limitations under the License.
- * //
- * //-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+ *******************************************************************
+ *
+ * Copyright 2015 Intel Corporation.
+ *
+ *-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
+ *
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
+ *
+ *      http://www.apache.org/licenses/LICENSE-2.0
+ *
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License.
+ *
+ *-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
  */
 
 package org.iotivity.base;
@@ -25,6 +25,7 @@ package org.iotivity.base;
 import android.test.InstrumentationTestCase;
 import android.util.Log;
 
+import java.util.Arrays;
 import java.util.Date;
 import java.util.EnumSet;
 import java.util.HashMap;
@@ -84,6 +85,11 @@ public class SmokeTest extends InstrumentationTestCase {
                         }
                         signal.countDown();
                     }
+
+                    @Override
+                    public void onFindResourceFailed(Throwable ex, String uri) {
+                        Log.i(TAG, "Find Resource Failed for Uri: " + uri + " Error: " + ex.getMessage());
+                    }
                 };
 
         try {
@@ -156,6 +162,11 @@ public class SmokeTest extends InstrumentationTestCase {
                             assertTrue(false);
                         }
                     }
+
+                    @Override
+                    public void onFindResourceFailed(Throwable ex, String uri) {
+                        Log.i(TAG, "Find Resource Failed for Uri: " + uri + " Error: " + ex.getMessage());
+                    }
                 };
 
         try {
@@ -175,6 +186,13 @@ public class SmokeTest extends InstrumentationTestCase {
                     resourceFoundListener);
 
             //server
+            //wait 2 seconds for the client's resourceFoundListener to set the presenceListener.
+            //the presenceListener must be set before startPresence() is called to get notified.
+            try {
+                Thread.sleep(2000);
+            } catch (InterruptedException e) {
+                Log.e(TAG, e.getMessage());
+            }
             OcPlatform.startPresence(OcPlatform.DEFAULT_PRESENCE_TTL);
 
             //wait for onPresence event
@@ -262,6 +280,11 @@ public class SmokeTest extends InstrumentationTestCase {
                         }
                         signal1.countDown();
                     }
+
+                    @Override
+                    public void onFindResourceFailed(Throwable ex, String uri) {
+                        Log.i(TAG, "Find Resource Failed for Uri: " + uri + " Error: " + ex.getMessage());
+                    }
                 };
 
         try {
@@ -429,6 +452,11 @@ public class SmokeTest extends InstrumentationTestCase {
                         }
                         signal1.countDown();
                     }
+
+                    @Override
+                    public void onFindResourceFailed(Throwable ex, String uri) {
+                        Log.i(TAG, "Find Resource Failed for Uri: " + uri + " Error: " + ex.getMessage());
+                    }
                 };
 
         try {
@@ -602,6 +630,11 @@ public class SmokeTest extends InstrumentationTestCase {
                         }
                         signal1.countDown();
                     }
+
+                    @Override
+                    public void onFindResourceFailed(Throwable ex, String uri) {
+                        Log.i(TAG, "Find Resource Failed for Uri: " + uri + " Error: " + ex.getMessage());
+                    }
                 };
 
         try {
@@ -730,6 +763,11 @@ public class SmokeTest extends InstrumentationTestCase {
                         }
                         signal1.countDown();
                     }
+
+                    @Override
+                    public void onFindResourceFailed(Throwable ex, String uri) {
+                        Log.i(TAG, "Find Resource Failed for Uri: " + uri + " Error: " + ex.getMessage());
+                    }
                 };
 
         try {
@@ -894,6 +932,11 @@ public class SmokeTest extends InstrumentationTestCase {
                         }
                         signal1.countDown();
                     }
+
+                    @Override
+                    public void onFindResourceFailed(Throwable ex, String uri) {
+                        Log.i(TAG, "Find Resource Failed for Uri: " + uri + " Error: " + ex.getMessage());
+                    }
                 };
 
         try {
@@ -1048,7 +1091,10 @@ public class SmokeTest extends InstrumentationTestCase {
             }
         };
 
-        OcDeviceInfo devInfo = new OcDeviceInfo("myDeviceName");
+        OcDeviceInfo devInfo = new OcDeviceInfo(
+                "myDeviceName",
+                Arrays.asList(new String[]{"oic.d.test"})
+        );
 
         try {
             //server
@@ -1090,6 +1136,11 @@ public class SmokeTest extends InstrumentationTestCase {
             public void onResourceFound(OcResource resource) {
                 signal1.countDown();
             }
+
+            @Override
+            public void onFindResourceFailed(Throwable ex, String uri) {
+                Log.i(TAG, "Find Resource Failed for Uri: " + uri + " Error: " + ex.getMessage());
+            }
         };
 
         OcPlatform.OnResourceFoundListener resourceFoundListener2 = new OcPlatform.OnResourceFoundListener() {
@@ -1097,6 +1148,11 @@ public class SmokeTest extends InstrumentationTestCase {
             public void onResourceFound(OcResource resource) {
                 signal2.countDown();
             }
+
+            @Override
+            public void onFindResourceFailed(Throwable ex, String uri) {
+                Log.i(TAG, "Find Resource Failed for Uri: " + uri + " Error: " + ex.getMessage());
+            }
         };
 
         OcPlatform.OnResourceFoundListener resourceFoundListener3 = new OcPlatform.OnResourceFoundListener() {
@@ -1104,6 +1160,11 @@ public class SmokeTest extends InstrumentationTestCase {
             public void onResourceFound(OcResource resource) {
                 signal3.countDown();
             }
+
+            @Override
+            public void onFindResourceFailed(Throwable ex, String uri) {
+                Log.i(TAG, "Find Resource Failed for Uri: " + uri + " Error: " + ex.getMessage());
+            }
         };
 
         OcPlatform.OnResourceFoundListener resourceFoundListener4 = new OcPlatform.OnResourceFoundListener() {
@@ -1111,6 +1172,11 @@ public class SmokeTest extends InstrumentationTestCase {
             public void onResourceFound(OcResource resource) {
                 signal4.countDown();
             }
+
+            @Override
+            public void onFindResourceFailed(Throwable ex, String uri) {
+                Log.i(TAG, "Find Resource Failed for Uri: " + uri + " Error: " + ex.getMessage());
+            }
         };
 
         OcPlatform.OnResourceFoundListener resourceFoundListener5 = new OcPlatform.OnResourceFoundListener() {
@@ -1118,6 +1184,11 @@ public class SmokeTest extends InstrumentationTestCase {
             public void onResourceFound(OcResource resource) {
                 signal5.countDown();
             }
+
+            @Override
+            public void onFindResourceFailed(Throwable ex, String uri) {
+                Log.i(TAG, "Find Resource Failed for Uri: " + uri + " Error: " + ex.getMessage());
+            }
         };
 
         OcPlatform.OnResourceFoundListener resourceFoundListener6 = new OcPlatform.OnResourceFoundListener() {
@@ -1125,6 +1196,11 @@ public class SmokeTest extends InstrumentationTestCase {
             public void onResourceFound(OcResource resource) {
                 signal6.countDown();
             }
+
+            @Override
+            public void onFindResourceFailed(Throwable ex, String uri) {
+                Log.i(TAG, "Find Resource Failed for Uri: " + uri + " Error: " + ex.getMessage());
+            }
         };
 
         try {
@@ -1266,7 +1342,12 @@ public class SmokeTest extends InstrumentationTestCase {
                         headerOptionList.add(new OcHeaderOption(2885, "OptionData1"));
                         headerOptionList.add(new OcHeaderOption(2886, "OptionData2"));
 
-                        resource.setHeaderOptions(headerOptionList);
+                        try {
+                            resource.setHeaderOptions(headerOptionList);
+                        } catch (OcException e) {
+                            Log.e(TAG, "onResourceFound, error in setHeaderOptions -- " + e.getMessage());
+                        }
+
                         resource.unsetHeaderOptions();
 
                         OcResourceIdentifier resourceIdentifier = resource.getUniqueIdentifier();
@@ -1275,6 +1356,11 @@ public class SmokeTest extends InstrumentationTestCase {
 
                         signal.countDown();
                     }
+
+                    @Override
+                    public void onFindResourceFailed(Throwable ex, String uri) {
+                        Log.i(TAG, "Find Resource Failed for Uri: " + uri + " Error: " + ex.getMessage());
+                    }
                 };
 
         try {
@@ -1355,6 +1441,11 @@ public class SmokeTest extends InstrumentationTestCase {
                         }
                         signal.countDown();
                     }
+
+                    @Override
+                    public void onFindResourceFailed(Throwable ex, String uri) {
+                        Log.i(TAG, "Find Resource Failed for Uri: " + uri + " Error: " + ex.getMessage());
+                    }
                 };
 
         try {
@@ -1553,6 +1644,11 @@ public class SmokeTest extends InstrumentationTestCase {
                             }
                         }
                     }
+
+                    @Override
+                    public void onFindResourceFailed(Throwable ex, String uri) {
+                        Log.i(TAG, "Find Resource Failed for Uri: " + uri + " Error: " + ex.getMessage());
+                    }
                 };
         try {
             //server
