@@ -24,6 +24,12 @@
 #include <stdlib.h>
 #include "oic_malloc.h"
 
+#include "iotivity_config.h"
+
+#ifdef HAVE_WINDOWS_H
+#include <windows.h>
+#endif
+
 // Enable extra debug logging for malloc.  Comment out to disable
 #ifdef ENABLE_MALLOC_DEBUG
 #include "logger.h"
@@ -134,3 +140,18 @@ void OICFree(void *ptr)
 
     free(ptr);
 }
+
+void OICClearMemory(void *buf, size_t n)
+{
+    if (NULL != buf) {
+#ifdef HAVE_WINDOWS_H
+        SecureZeroMemory(buf, n);
+#else
+        volatile unsigned char* p = (volatile unsigned char*)buf;
+        while (n--)
+        {
+            *p++ = 0;
+        }
+#endif
+    }
+}
\ No newline at end of file
