@@ -27,13 +27,20 @@
 #ifndef _GNU_SOURCE
 #define _GNU_SOURCE
 #endif
+#include "iotivity_config.h"
 #include <errno.h>
+#if defined HAVE_PTHREAD_H
 #include <pthread.h>
+#endif
+#if defined HAVE_WINSOCK2_H
+#include <winsock2.h>
+#endif
 #include "cathreadpool.h"
 #include "logger.h"
 #include "oic_malloc.h"
 #include "uarraylist.h"
 #include "camutex.h"
+#include "platform_features.h"
 
 #define TAG PCF("UTHREADPOOL")
 
@@ -109,10 +116,7 @@ CAResult_t ca_thread_pool_init(int32_t num_of_threads, ca_thread_pool_t *thread_
     if(!(*thread_pool)->details->list_lock)
     {
         OIC_LOG(ERROR, TAG, "Failed to create thread-pool mutex");
-        OICFree((*thread_pool)->details);
-        OICFree(*thread_pool);
-        *thread_pool = NULL;
-        return CA_STATUS_FAILED;
+        goto exit;
     }
 
     (*thread_pool)->details->threads_list = u_arraylist_create();
@@ -124,15 +128,17 @@ CAResult_t ca_thread_pool_init(int32_t num_of_threads, ca_thread_pool_t *thread_
         {
             OIC_LOG(ERROR, TAG, "Failed to free thread-pool mutex");
         }
-
-        OICFree((*thread_pool)->details);
-        OICFree(*thread_pool);
-        *thread_pool = NULL;
-        return CA_STATUS_FAILED;
+        goto exit;
     }
 
     OIC_LOG(DEBUG, TAG, "OUT");
     return CA_STATUS_OK;
+
+exit:
+    OICFree((*thread_pool)->details);
+    OICFree(*thread_pool);
+    *thread_pool = NULL;
+    return CA_STATUS_FAILED;
 }
 
 CAResult_t ca_thread_pool_add_task(ca_thread_pool_t thread_pool, ca_thread_func method,
@@ -157,7 +163,6 @@ CAResult_t ca_thread_pool_add_task(ca_thread_pool_t thread_pool, ca_thread_func
     info->data = data;
 
     pthread_t threadHandle;
-
     int result = pthread_create(&threadHandle, NULL, ca_thread_pool_pthreads_delegate, info);
 
     if(result != 0)
@@ -170,9 +175,23 @@ CAResult_t ca_thread_pool_add_task(ca_thread_pool_t thread_pool, ca_thread_func
     bool addResult = u_arraylist_add(thread_pool->details->threads_list, (void*)threadHandle);
     ca_mutex_unlock(thread_pool->details->list_lock);
 
-    if(!addResult)
+    if (!addResult)
     {
         OIC_LOG_V(ERROR, TAG, "Arraylist Add failed, may not be properly joined: %d", addResult);
+#if defined(_WIN32)
+        DWORD joinres = WaitForSingleObject(threadHandle, INFINITE);
+        if (WAIT_OBJECT_0 != joinres)
+        {
+            OIC_LOG_V(ERROR, TAG, "Failed to join thread with error %d", joinres);
+        }
+        CloseHandle(threadHandle);
+#else
+        int joinres = pthread_join(threadHandle, NULL);
+        if (0 != joinres)
+        {
+            OIC_LOG_V(ERROR, TAG, "Failed to join thread with error %d", joinres);
+        }
+#endif
         return CA_STATUS_FAILED;
     }
 
@@ -195,11 +214,20 @@ void ca_thread_pool_free(ca_thread_pool_t thread_pool)
     for(uint32_t i = 0; i<u_arraylist_length(thread_pool->details->threads_list); ++i)
     {
         pthread_t tid = (pthread_t)u_arraylist_get(thread_pool->details->threads_list, i);
+#if defined(_WIN32)
+        DWORD joinres = WaitForSingleObject(tid, INFINITE);
+        if (WAIT_OBJECT_0 != joinres)
+        {
+            OIC_LOG_V(ERROR, TAG, "Failed to join thread at index %u with error %d", i, joinres);
+        }
+        CloseHandle(tid);
+#else
         int joinres = pthread_join(tid, NULL);
         if(0 != joinres)
         {
             OIC_LOG_V(ERROR, TAG, "Failed to join thread at index %u with error %d", i, joinres);
         }
+#endif
     }
 
     u_arraylist_free(&(thread_pool->details->threads_list));
