@@ -51,7 +51,7 @@ extern "C"
  */
 CAResult_t CAInitializeRA(CARegisterConnectivityCallback registerCallback,
                           CANetworkPacketReceivedCallback networkPacketCallback,
-                          CANetworkChangeCallback netCallback,
+                          CAAdapterChangeCallback netCallback,
                           ca_thread_pool_t handle);
 
 
@@ -68,11 +68,12 @@ CAResult_t CAStartRA();
  *                           the unicast data has to be sent.
  * @param[in]   data        Data which is required to be sent.
  * @param[in]   dataLen     Size of data to be sent.
+ * @param[in]   dataType    Data type which is REQUEST or RESPONSE.
  * @note dataLen must be > 0.
  * @return The number of bytes sent on the network, or -1 upon error.
  */
 int32_t CASendRAUnicastData(const CAEndpoint_t *endpoint, const void *data,
-                            uint32_t dataLen);
+                            uint32_t dataLen, CADataType_t dataType);
 
 /**
  * Get RA Connectivity network information.
@@ -107,7 +108,7 @@ CAResult_t CASetRAInfo(const CARAInfo_t *caraInfo);
  * These functions are not applicable to Remote Access adapter.
  */
 int32_t CASendRAMulticastData(const CAEndpoint_t *endpoint,
-                 const void *data, uint32_t dataLen);
+                              const void *data, uint32_t dataLen, CADataType_t dataType);
 
 /**
  * Start listening server for receiving search requests.
