@@ -27,6 +27,7 @@
 #include "oic_string.h"
 #include "cathreadpool.h"
 #include "uarraylist.h"
+#include "caadapterutils.h"
 
 #define ERROR_CODE (-1)
 #define TAG PCF("OIC_CA_EDR_UTILS")
@@ -43,25 +44,13 @@ static u_arraylist_t *g_deviceObjectList = NULL;
 // get address from bluetooth socket
 jstring CAEDRNativeGetAddressFromDeviceSocket(JNIEnv *env, jobject bluetoothSocketObj)
 {
-    if (!bluetoothSocketObj)
-    {
-        OIC_LOG(ERROR, TAG, "bluetoothSocketObj is null");
-        return NULL;
-    }
+    VERIFY_NON_NULL_RET(bluetoothSocketObj, TAG, "bluetoothSocketObj", NULL);
 
-    jclass jni_cid_BTSocket = (*env)->FindClass(env, CLASSPATH_BT_SOCKET);
-    if (!jni_cid_BTSocket)
-    {
-        OIC_LOG(ERROR, TAG, "jni_cid_BTSocket is null");
-        return NULL;
-    }
-
-    jmethodID jni_mid_getRemoteDevice = (*env)->GetMethodID(
-            env, jni_cid_BTSocket, "getRemoteDevice", "()Landroid/bluetooth/BluetoothDevice;");
+    jmethodID jni_mid_getRemoteDevice = CAGetJNIMethodID(
+            env, CLASSPATH_BT_SOCKET, "getRemoteDevice", "()Landroid/bluetooth/BluetoothDevice;");
     if (!jni_mid_getRemoteDevice)
     {
         OIC_LOG(ERROR, TAG, "jni_mid_getRemoteDevice is null");
-        (*env)->DeleteLocalRef(env, jni_cid_BTSocket);
         return NULL;
     }
 
@@ -70,42 +59,29 @@ jstring CAEDRNativeGetAddressFromDeviceSocket(JNIEnv *env, jobject bluetoothSock
     if (!jni_obj_remoteBTDevice)
     {
         OIC_LOG(ERROR, TAG, "jni_obj_remoteBTDevice is null");
-        (*env)->DeleteLocalRef(env, jni_cid_BTSocket);
         return NULL;
     }
 
-    jclass jni_cid_BTDevice = (*env)->FindClass(env, CLASSPATH_BT_DEVICE);
-    if (!jni_cid_BTDevice)
-    {
-        OIC_LOG(ERROR, TAG, "jni_cid_BTDevice is null");
-        (*env)->DeleteLocalRef(env, jni_obj_remoteBTDevice);
-        (*env)->DeleteLocalRef(env, jni_cid_BTSocket);
-        return NULL;
-    }
-    jmethodID j_mid_getAddress = (*env)->GetMethodID(env, jni_cid_BTDevice, "getAddress",
-                                                     METHODID_STRINGNONPARAM);
+    jmethodID j_mid_getAddress = CAGetJNIMethodID(env, CLASSPATH_BT_DEVICE,
+                                                  "getAddress",
+                                                  METHODID_STRINGNONPARAM);
     if (!j_mid_getAddress)
     {
         OIC_LOG(ERROR, TAG, "j_mid_getAddress is null");
         (*env)->DeleteLocalRef(env, jni_obj_remoteBTDevice);
-        (*env)->DeleteLocalRef(env, jni_cid_BTDevice);
-        (*env)->DeleteLocalRef(env, jni_cid_BTSocket);
         return NULL;
     }
 
-    jstring j_str_address = (*env)->CallObjectMethod(env, jni_obj_remoteBTDevice, j_mid_getAddress);
+    jstring j_str_address = (*env)->CallObjectMethod(env, jni_obj_remoteBTDevice,
+                                                     j_mid_getAddress);
     if (!j_str_address)
     {
         OIC_LOG(ERROR, TAG, "j_str_address is null");
         (*env)->DeleteLocalRef(env, jni_obj_remoteBTDevice);
-        (*env)->DeleteLocalRef(env, jni_cid_BTDevice);
-        (*env)->DeleteLocalRef(env, jni_cid_BTSocket);
         return NULL;
     }
 
     (*env)->DeleteLocalRef(env, jni_obj_remoteBTDevice);
-    (*env)->DeleteLocalRef(env, jni_cid_BTDevice);
-    (*env)->DeleteLocalRef(env, jni_cid_BTSocket);
 
     return j_str_address;
 }
@@ -215,15 +191,8 @@ jobjectArray CAEDRNativeGetBondedDevices(JNIEnv *env)
 
     // Convert the set to an object array
     // object[] array = Set<BluetoothDevice>.toArray();
-    jclass jni_cid_Set = (*env)->FindClass(env, "java/util/Set");
-    if (!jni_cid_Set)
-    {
-        OIC_LOG(ERROR, TAG, "jni_cid_Set is null");
-        goto exit;
-    }
-    jmethodID jni_mid_toArray = (*env)->GetMethodID(env, jni_cid_Set, "toArray",
-                                                    "()[Ljava/lang/Object;");
-
+    jmethodID jni_mid_toArray = CAGetJNIMethodID(env, "java/util/Set",
+                                                 "toArray", "()[Ljava/lang/Object;");
     if (!jni_mid_toArray)
     {
         OIC_LOG(ERROR, TAG, "jni_mid_toArray is null");
@@ -325,20 +294,12 @@ jboolean CAEDRNativeIsEnableBTAdapter(JNIEnv *env)
 
 jstring CAEDRNativeGetAddressFromBTDevice(JNIEnv *env, jobject bluetoothDevice)
 {
-    if (!bluetoothDevice)
-    {
-        OIC_LOG(ERROR, TAG, "bluetoothDevice is null");
-        return NULL;
-    }
-    jclass jni_cid_device_list = (*env)->FindClass(env, "android/bluetooth/BluetoothDevice");
-    if (!jni_cid_device_list)
-    {
-        OIC_LOG(ERROR, TAG, "jni_cid_device_list is null");
-        return NULL;
-    }
+    VERIFY_NON_NULL_RET(bluetoothDevice, TAG, "bluetoothDevice", NULL);
 
-    jmethodID jni_mid_getAddress = (*env)->GetMethodID(env, jni_cid_device_list, "getAddress",
-                                                       METHODID_STRINGNONPARAM);
+    jmethodID jni_mid_getAddress = CAGetJNIMethodID(env,
+                                                    CLASSPATH_BT_DEVICE,
+                                                    "getAddress",
+                                                    METHODID_STRINGNONPARAM);
     if (!jni_mid_getAddress)
     {
         OIC_LOG(ERROR, TAG, "jni_mid_getAddress is null");
@@ -373,11 +334,8 @@ void CAEDRNativeCreateDeviceStateList()
 
 void CAEDRUpdateDeviceState(CAConnectedState_t state, const char *address)
 {
-    if (!address)
-    {
-        OIC_LOG(ERROR, TAG, "address is null");
-        return;
-    }
+    VERIFY_NON_NULL_VOID(address, TAG, "address");
+
     CAConnectedDeviceInfo_t *deviceInfo =
             (CAConnectedDeviceInfo_t *) OICCalloc(1, sizeof(CAConnectedDeviceInfo_t));
     if (!deviceInfo)
@@ -393,11 +351,7 @@ void CAEDRUpdateDeviceState(CAConnectedState_t state, const char *address)
 
 void CAEDRNativeAddDeviceStateToList(CAConnectedDeviceInfo_t *deviceInfo)
 {
-    if (!deviceInfo)
-    {
-        OIC_LOG(ERROR, TAG, "device is null");
-        return;
-    }
+    VERIFY_NON_NULL_VOID(deviceInfo, TAG, "deviceInfo");
 
     if (!g_deviceStateList)
     {
@@ -411,16 +365,12 @@ void CAEDRNativeAddDeviceStateToList(CAConnectedDeviceInfo_t *deviceInfo)
         CAEDRNativeRemoveDevice((const char*) deviceInfo->address);
     }
     u_arraylist_add(g_deviceStateList, deviceInfo); // update new state
-    OIC_LOG_V(DEBUG, TAG, "Set State Info to List : %d", deviceInfo->state);
+    OIC_LOG_V(DEBUG, TAG, "add new device state[%d] to list", deviceInfo->state);
 }
 
 bool CAEDRNativeIsDeviceInList(const char* remoteAddress)
 {
-    if (!remoteAddress)
-    {
-        OIC_LOG(ERROR, TAG, "remoteAddress is null");
-        return false;
-    }
+    VERIFY_NON_NULL_RET(remoteAddress, TAG, "remoteAddress", false);
 
     jint length = u_arraylist_length(g_deviceStateList);
     for (jint index = 0; index < length; index++)
@@ -476,17 +426,13 @@ void CAEDRNativeRemoveAllDeviceState()
 void CAEDRNativeRemoveDevice(const char *remoteAddress)
 {
     OIC_LOG(DEBUG, TAG, "CAEDRNativeRemoveDeviceforStateList");
+    VERIFY_NON_NULL_VOID(remoteAddress, TAG, "remoteAddress");
 
     if (!g_deviceStateList)
     {
         OIC_LOG(ERROR, TAG, "gdeviceStateList is null");
         return;
     }
-    if (!remoteAddress)
-    {
-        OIC_LOG(ERROR, TAG, "remoteAddress is null");
-        return;
-    }
 
     jint length = u_arraylist_length(g_deviceStateList);
     for (jint index = 0; index < length; index++)
@@ -501,7 +447,7 @@ void CAEDRNativeRemoveDevice(const char *remoteAddress)
 
         if (!strcmp((const char*) deviceInfo->address, remoteAddress))
         {
-            OIC_LOG_V(DEBUG, TAG, "remove state : %s", remoteAddress);
+            OIC_LOG_V(DEBUG, TAG, "remove [%s] info from list", remoteAddress);
             OICFree(deviceInfo);
 
             u_arraylist_remove(g_deviceStateList, index);
@@ -514,12 +460,7 @@ void CAEDRNativeRemoveDevice(const char *remoteAddress)
 CAConnectedState_t CAEDRIsConnectedDevice(const char *remoteAddress)
 {
     OIC_LOG(DEBUG, TAG, "CAEDRIsConnectedDevice");
-
-    if (!remoteAddress)
-    {
-        OIC_LOG(ERROR, TAG, "remoteAddress is null");
-        return STATE_DISCONNECTED;
-    }
+    VERIFY_NON_NULL_RET(remoteAddress, TAG, "remoteAddress", STATE_DISCONNECTED);
 
     if (!g_deviceStateList)
     {
@@ -565,12 +506,7 @@ void CAEDRNativeCreateDeviceSocketList()
 void CAEDRNativeAddDeviceSocketToList(JNIEnv *env, jobject deviceSocket)
 {
     OIC_LOG(DEBUG, TAG, "CANativeAddDeviceobjToList");
-
-    if (!deviceSocket)
-    {
-        OIC_LOG(ERROR, TAG, "Device is null");
-        return;
-    }
+    VERIFY_NON_NULL_VOID(deviceSocket, TAG, "deviceSocket");
 
     if (!g_deviceObjectList)
     {
@@ -587,31 +523,71 @@ void CAEDRNativeAddDeviceSocketToList(JNIEnv *env, jobject deviceSocket)
 
     const char* remoteAddress = (*env)->GetStringUTFChars(env, jni_remoteAddress, NULL);
 
-    if (!CAEDRNativeIsDeviceSocketInList(env, remoteAddress))
+    if (CAEDRNativeIsDeviceSocketInList(env, remoteAddress))
     {
-        jobject gDeviceSocker = (*env)->NewGlobalRef(env, deviceSocket);
-        u_arraylist_add(g_deviceObjectList, gDeviceSocker);
-        OIC_LOG(DEBUG, TAG, "Set Socket Object to Array");
+        OIC_LOG(DEBUG, TAG, "the address exists in deviceObjectList. remove it to add new");
+        CAEDRNativeRemoveDeviceSocketBaseAddr(env, jni_remoteAddress);
     }
+
     (*env)->ReleaseStringUTFChars(env, jni_remoteAddress, remoteAddress);
     (*env)->DeleteLocalRef(env, jni_remoteAddress);
+
+    CAEDRSocketInfo_t *socketInfo = (CAEDRSocketInfo_t *) OICCalloc(1, sizeof(*socketInfo));
+    if (!socketInfo)
+    {
+        OIC_LOG(ERROR, TAG, "Out of memory");
+        return;
+    }
+
+    jmethodID jni_mid_getInputStream = CAGetJNIMethodID(env, "android/bluetooth/BluetoothSocket",
+                                                             "getInputStream",
+                                                             "()Ljava/io/InputStream;");
+    if (!jni_mid_getInputStream)
+    {
+        OIC_LOG(ERROR, TAG, "jni_mid_getInputStream is null");
+        return;
+    }
+
+    jobject jni_obj_inputStream = (*env)->CallObjectMethod(env, deviceSocket,
+                                                           jni_mid_getInputStream);
+    if (!jni_obj_inputStream)
+    {
+        OIC_LOG(ERROR, TAG, "jni_obj_inputStream is null");
+        return;
+    }
+
+    socketInfo->deviceSocket = (*env)->NewGlobalRef(env, deviceSocket);
+    socketInfo->inputStream = (*env)->NewGlobalRef(env, jni_obj_inputStream);
+    (*env)->DeleteLocalRef(env, jni_obj_inputStream);
+
+    bool result = u_arraylist_add(g_deviceObjectList, (void *) socketInfo);
+    if (!result)
+    {
+        OIC_LOG(ERROR, TAG, "u_arraylist_add failed.");
+        OICFree(socketInfo);
+        return;
+    }
+
+    OIC_LOG(DEBUG, TAG, "add new device socket object to list");
 }
 
 bool CAEDRNativeIsDeviceSocketInList(JNIEnv *env, const char* remoteAddress)
 {
     OIC_LOG(DEBUG, TAG, "CANativeIsDeviceObjInList");
+    VERIFY_NON_NULL_RET(remoteAddress, TAG, "remoteAddress", false);
 
-    if (!remoteAddress)
-    {
-        OIC_LOG(ERROR, TAG, "remoteAddress is null");
-        return false;
-    }
-
-    jint length = u_arraylist_length(g_deviceStateList);
+    jint length = u_arraylist_length(g_deviceObjectList);
     for (jint index = 0; index < length; index++)
     {
+        CAEDRSocketInfo_t *socketInfo = (CAEDRSocketInfo_t *) u_arraylist_get(g_deviceObjectList,
+                                                                              index);
+        if (!socketInfo)
+        {
+            OIC_LOG(DEBUG, TAG, "socketInfo is null");
+            return false;
+        }
 
-        jobject jarrayObj = (jobject) u_arraylist_get(g_deviceObjectList, index);
+        jobject jarrayObj = socketInfo->deviceSocket;
         if (!jarrayObj)
         {
             OIC_LOG(DEBUG, TAG, "jarrayObj is null");
@@ -645,7 +621,7 @@ bool CAEDRNativeIsDeviceSocketInList(JNIEnv *env, const char* remoteAddress)
         }
     }
 
-    OIC_LOG(DEBUG, TAG, "there are no the Device obejct in list. we can add");
+    OIC_LOG(DEBUG, TAG, "there are no the Device obejct in list");
     return false;
 }
 
@@ -659,21 +635,15 @@ void CAEDRNativeSocketCloseToAll(JNIEnv *env)
         return;
     }
 
-    jclass jni_cid_BTSocket = (*env)->FindClass(env, CLASSPATH_BT_SOCKET);
-    if (!jni_cid_BTSocket)
-    {
-        OIC_LOG(ERROR, TAG, "jni_cid_BTSocket is null");
-        return;
-    }
-
-    jmethodID jni_mid_close = (*env)->GetMethodID(env, jni_cid_BTSocket, "close", "()V");
+    jmethodID jni_mid_close = CAGetJNIMethodID(env, CLASSPATH_BT_SOCKET,
+                                               "close", "()V");
     if (!jni_mid_close)
     {
         OIC_LOG(ERROR, TAG, "jni_mid_close is null");
         return;
     }
 
-    jint length = u_arraylist_length(g_deviceStateList);
+    jint length = u_arraylist_length(g_deviceObjectList);
     for (jint index = 0; index < length; index++)
     {
         jobject jni_obj_socket = (jobject) u_arraylist_get(g_deviceObjectList, index);
@@ -705,16 +675,28 @@ void CAEDRNativeRemoveAllDeviceSocket(JNIEnv *env)
         return;
     }
 
-    jint length = u_arraylist_length(g_deviceStateList);
+    jint length = u_arraylist_length(g_deviceObjectList);
     for (jint index = 0; index < length; index++)
     {
-        jobject jarrayObj = (jobject) u_arraylist_get(g_deviceObjectList, index);
-        if (!jarrayObj)
+        CAEDRSocketInfo_t *socketInfo = (CAEDRSocketInfo_t *) u_arraylist_get(g_deviceObjectList,
+                                                                              index);
+        if (!socketInfo)
         {
-            OIC_LOG(ERROR, TAG, "jarrayObj is null");
-            return;
+            OIC_LOG(ERROR, TAG, "socketInfo is null");
+            continue;
+        }
+
+        jobject jdeviceSocket = socketInfo->deviceSocket;
+        if (jdeviceSocket)
+        {
+            (*env)->DeleteGlobalRef(env, jdeviceSocket);
+        }
+
+        jobject jinputStream = socketInfo->inputStream;
+        if (jinputStream)
+        {
+            (*env)->DeleteGlobalRef(env, jinputStream);
         }
-        (*env)->DeleteGlobalRef(env, jarrayObj);
     }
 
     OICFree(g_deviceObjectList);
@@ -725,61 +707,24 @@ void CAEDRNativeRemoveAllDeviceSocket(JNIEnv *env)
 void CAEDRNativeRemoveDeviceSocket(JNIEnv *env, jobject deviceSocket)
 {
     OIC_LOG(DEBUG, TAG, "CAEDRNativeRemoveDeviceSocket");
+    VERIFY_NON_NULL_VOID(deviceSocket, TAG, "deviceSocket");
 
-    if (!g_deviceObjectList)
+    jstring jni_address = CAEDRNativeGetAddressFromDeviceSocket(env, deviceSocket);
+    if (!jni_address)
     {
-        OIC_LOG(ERROR, TAG, "gdeviceObjectList is null");
+        OIC_LOG(ERROR, TAG, "jni_address is null");
         return;
     }
 
-    jint length = u_arraylist_length(g_deviceStateList);
-    for (jint index = 0; index < length; index++)
-    {
-        jobject jarrayObj = (jobject) u_arraylist_get(g_deviceObjectList, index);
-        if (!jarrayObj)
-        {
-            OIC_LOG(DEBUG, TAG, "jarrayObj is null");
-            continue;
-        }
+    CAEDRNativeRemoveDeviceSocketBaseAddr(env, jni_address);
 
-        jstring jni_setAddress = CAEDRNativeGetAddressFromDeviceSocket(env, jarrayObj);
-        if (!jni_setAddress)
-        {
-            OIC_LOG(DEBUG, TAG, "jni_setAddress is null");
-            continue;
-        }
-
-        jstring jni_remoteAddress = CAEDRNativeGetAddressFromDeviceSocket(env, deviceSocket);
-        if (!jni_remoteAddress)
-        {
-            OIC_LOG(DEBUG, TAG, "jni_remoteAddress is null");
-            continue;
-        }
-
-        const char* setAddress = (*env)->GetStringUTFChars(env, jni_setAddress, NULL);
-        const char* remoteAddress = (*env)->GetStringUTFChars(env, jni_remoteAddress, NULL);
-
-        if (!strcmp(setAddress, remoteAddress))
-        {
-            OIC_LOG_V(DEBUG, TAG, "remove object : %s", remoteAddress);
-            (*env)->DeleteGlobalRef(env, jarrayObj);
-            (*env)->ReleaseStringUTFChars(env, jni_setAddress, setAddress);
-            (*env)->ReleaseStringUTFChars(env, jni_remoteAddress, remoteAddress);
-
-            u_arraylist_remove(g_deviceObjectList, index);
-            break;
-        }
-        (*env)->ReleaseStringUTFChars(env, jni_setAddress, setAddress);
-        (*env)->ReleaseStringUTFChars(env, jni_remoteAddress, remoteAddress);
-    }
-
-    OIC_LOG(DEBUG, TAG, "there are no target object");
     return;
 }
 
 void CAEDRNativeRemoveDeviceSocketBaseAddr(JNIEnv *env, jstring address)
 {
-    OIC_LOG(DEBUG, TAG, "CAEDRNativeRemoveDeviceSocket");
+    OIC_LOG(DEBUG, TAG, "CAEDRNativeRemoveDeviceSocketBaseAddr");
+    VERIFY_NON_NULL_VOID(address, TAG, "address");
 
     if (!g_deviceObjectList)
     {
@@ -787,10 +732,20 @@ void CAEDRNativeRemoveDeviceSocketBaseAddr(JNIEnv *env, jstring address)
         return;
     }
 
-    jint length = u_arraylist_length(g_deviceStateList);
+    const char* targetAddress = (*env)->GetStringUTFChars(env, address, NULL);
+
+    jint length = u_arraylist_length(g_deviceObjectList);
     for (jint index = 0; index < length; index++)
     {
-        jobject jarrayObj = (jobject) u_arraylist_get(g_deviceObjectList, index);
+        CAEDRSocketInfo_t *socketInfo = (CAEDRSocketInfo_t *) u_arraylist_get(g_deviceObjectList,
+                                                                              index);
+        if (!socketInfo)
+        {
+            OIC_LOG(ERROR, TAG, "socketInfo is null");
+            continue;
+        }
+
+        jobject jarrayObj = socketInfo->deviceSocket;
         if (!jarrayObj)
         {
             OIC_LOG(DEBUG, TAG, "jarrayObj is null");
@@ -803,28 +758,37 @@ void CAEDRNativeRemoveDeviceSocketBaseAddr(JNIEnv *env, jstring address)
             OIC_LOG(ERROR, TAG, "jni_setAddress is null");
             continue;
         }
-        const char* setAddress = (*env)->GetStringUTFChars(env, jni_setAddress, NULL);
-        const char* remoteAddress = (*env)->GetStringUTFChars(env, address, NULL);
 
-        if (!strcmp(setAddress, remoteAddress))
+        const char* setAddress = (*env)->GetStringUTFChars(env, jni_setAddress, NULL);
+
+        if (!strcmp(setAddress, targetAddress))
         {
-            OIC_LOG_V(ERROR, TAG, "remove object : %s", remoteAddress);
+            OIC_LOG_V(DEBUG, TAG, "remove object : %s", targetAddress);
             (*env)->DeleteGlobalRef(env, jarrayObj);
+            jobject jinputStream = socketInfo->inputStream;
+            if (jinputStream)
+            {
+                (*env)->DeleteGlobalRef(env, jinputStream);
+            }
+            (*env)->ReleaseStringUTFChars(env, address, targetAddress);
             (*env)->ReleaseStringUTFChars(env, jni_setAddress, setAddress);
-            (*env)->ReleaseStringUTFChars(env, address, remoteAddress);
+            (*env)->DeleteLocalRef(env, jni_setAddress);
 
             u_arraylist_remove(g_deviceObjectList, index);
-            break;
+            return;
         }
         (*env)->ReleaseStringUTFChars(env, jni_setAddress, setAddress);
-        (*env)->ReleaseStringUTFChars(env, address, remoteAddress);
+        (*env)->DeleteLocalRef(env, jni_setAddress);
     }
 
-    OIC_LOG(DEBUG, TAG, "there are no target object");
+    OIC_LOG_V(DEBUG, TAG, "the target object doesn't exist in deviceObjectList (addr: %s)",
+              targetAddress);
+    (*env)->ReleaseStringUTFChars(env, address, targetAddress);
+
     return;
 }
 
-jobject CAEDRNativeGetDeviceSocket(uint32_t idx)
+jobject CAEDRNativeGetDeviceSocket(uint32_t index)
 {
     if (!g_deviceObjectList)
     {
@@ -832,7 +796,15 @@ jobject CAEDRNativeGetDeviceSocket(uint32_t idx)
         return NULL;
     }
 
-    jobject jarrayObj = (jobject) u_arraylist_get(g_deviceObjectList, idx);
+    CAEDRSocketInfo_t *socketInfo = (CAEDRSocketInfo_t *) u_arraylist_get(g_deviceObjectList,
+                                                                          index);
+    if (!socketInfo)
+    {
+        OIC_LOG(ERROR, TAG, "socketInfo is null");
+        return NULL;
+    }
+
+    jobject jarrayObj = socketInfo->deviceSocket;
     if (!jarrayObj)
     {
         OIC_LOG(ERROR, TAG, "jarrayObj is not available");
@@ -843,7 +815,8 @@ jobject CAEDRNativeGetDeviceSocket(uint32_t idx)
 
 jobject CAEDRNativeGetDeviceSocketBaseAddr(JNIEnv *env, const char* remoteAddress)
 {
-    OIC_LOG(DEBUG, TAG, "CAEDRNativeGetDeviceSocket");
+    OIC_LOG(DEBUG, TAG, "CAEDRNativeGetDeviceSocketBaseAddr");
+    VERIFY_NON_NULL_RET(remoteAddress, TAG, "remoteAddress", NULL);
 
     if (!g_deviceObjectList)
     {
@@ -851,10 +824,18 @@ jobject CAEDRNativeGetDeviceSocketBaseAddr(JNIEnv *env, const char* remoteAddres
         return NULL;
     }
 
-    jint length = u_arraylist_length(g_deviceStateList);
+    jint length = u_arraylist_length(g_deviceObjectList);
     for (jint index = 0; index < length; index++)
     {
-        jobject jarrayObj = (jobject) u_arraylist_get(g_deviceObjectList, index);
+        CAEDRSocketInfo_t *socketInfo = (CAEDRSocketInfo_t *) u_arraylist_get(g_deviceObjectList,
+                                                                              index);
+        if (!socketInfo)
+        {
+            OIC_LOG(ERROR, TAG, "socketInfo is null");
+            continue;
+        }
+
+        jobject jarrayObj = socketInfo->deviceSocket;
         if (!jarrayObj)
         {
             OIC_LOG(ERROR, TAG, "jarrayObj is null");
@@ -883,6 +864,31 @@ jobject CAEDRNativeGetDeviceSocketBaseAddr(JNIEnv *env, const char* remoteAddres
     return NULL;
 }
 
+jobject CAEDRNativeGetInputStream(uint32_t index)
+{
+    if (!g_deviceObjectList)
+    {
+        OIC_LOG(ERROR, TAG, "gdeviceObjectList is null");
+        return NULL;
+    }
+
+    CAEDRSocketInfo_t *socketInfo = (CAEDRSocketInfo_t *) u_arraylist_get(g_deviceObjectList,
+                                                                          index);
+    if (!socketInfo)
+    {
+        OIC_LOG(ERROR, TAG, "socketInfo is null");
+        return NULL;
+    }
+
+    jobject jarrayObj = socketInfo->inputStream;
+    if (!jarrayObj)
+    {
+        OIC_LOG(ERROR, TAG, "jarrayObj is not available");
+        return NULL;
+    }
+    return jarrayObj;
+}
+
 uint32_t CAEDRGetSocketListLength()
 {
     if (!g_deviceObjectList)
@@ -891,25 +897,19 @@ uint32_t CAEDRGetSocketListLength()
         return 0;
     }
 
-    uint32_t length = u_arraylist_length(g_deviceObjectList);
-
-    return length;
+    return u_arraylist_length(g_deviceObjectList);
 }
 
 CAConnectedDeviceInfo_t *CAEDRGetDeviceInfoFromAddress(const char *remoteAddress)
 {
     OIC_LOG(DEBUG, TAG, "CAEDRGetDeviceInfoFromAddress");
+    VERIFY_NON_NULL_RET(remoteAddress, TAG, "remoteAddress", NULL);
 
     if (!g_deviceStateList)
     {
         OIC_LOG(ERROR, TAG, "gdeviceStateList is null");
         return NULL;
     }
-    if (!remoteAddress)
-    {
-        OIC_LOG(ERROR, TAG, "remoteAddress is null");
-        return NULL;
-    }
 
     jint length = u_arraylist_length(g_deviceStateList);
     for (jint index = 0; index < length; index++)
