@@ -36,6 +36,11 @@ extern "C"
 {
 #endif
 
+#define CA_LE_AUTO_CONNECT_FLAG    1
+#define CA_LE_CONNECTION_STATE     2
+#define CA_LE_SEND_STATE           3
+#define CA_LE_DESCRIPTOR_FOUND     4
+
 /* Service UUID */
 static const char OIC_GATT_SERVICE_UUID[] = CA_GATT_SERVICE_UUID;
 static const char OIC_GATT_CHARACTERISTIC_REQUEST_UUID[] = CA_GATT_REQUEST_CHRC_UUID;
@@ -60,20 +65,9 @@ static const jint BOND_BONDED = 12;
 static const jint BOND_BONDING = 11;
 static const jint BOND_NONE = 10;
 
-static const jint STATE_CONNECTED = 2;
-static const jint STATE_DISCONNECTED = 0;
-
-/**
- * get method ID for method Name and class
- * @param[in]   env              JNI interface pointer.
- * @param[in]   className        android class.
- * @param[in]   methodName       android method name.
- * @param[in]   methodFormat     method type of methodName.
- * @return  jmethodID of the method.
- */
-jmethodID CALEGetJNIMethodID(JNIEnv *env, const char* className,
-                             const char* methodName,
-                             const char* methodFormat);
+static const uint16_t STATE_CONNECTED = 3;
+static const uint16_t STATE_SERVICE_CONNECTED = 2;
+static const uint16_t STATE_DISCONNECTED = 1;
 
 /**
  * get uuid(jni object) from uuid(character).
@@ -167,6 +161,14 @@ jint CALEGetConstantsValue(JNIEnv *env, const char* classType, const char* name)
  */
 jobject CALEGetRemoteDevice(JNIEnv *env, jstring address);
 
+/**
+ * get address from gatt profile object.
+ * @param[in]   env                   JNI interface pointer.
+ * @param[in]   gatt                  gatt profile object.
+ * @return  LE address.
+ */
+jstring CALEGetAddressFromGatt(JNIEnv *env, jobject gatt);
+
 #ifdef __cplusplus
 } /* extern "C" */
 #endif
