@@ -30,21 +30,26 @@
 #define _POSIX_C_SOURCE 200809L
 #endif
 
-// Platform check can be extended to check and/or define more, or could be
-// moved into a config.h
-#if !defined(__ARDUINO__) && !defined(ARDUINO)
-#define HAVE_UNISTD_H 1
-#endif
+#include "iotivity_config.h"
 
 // Pull in _POSIX_TIMERS feature test macro to check for
 // clock_gettime() support.
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
 
-// if we have unistd.h, we're a Unix system
+#ifdef HAVE_ARDUINO_TIME_H
+#include <Time.h>
+#else
 #include <time.h>
+#endif
+
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
+#ifdef HAVE_WINDOWS_H
+#include <windows.h>
+#endif
 
 #include "logger.h"
 #include "string.h"
@@ -53,15 +58,19 @@
 #ifndef __TIZEN__
 static oc_log_ctx_t *logCtx = 0;
 #endif
+
+#if defined(_MSC_VER)
+#define LINE_BUFFER_SIZE (16 * 2) + 16 + 1  // Show 16 bytes, 2 chars/byte, spaces between bytes, null termination
+#else
+static const uint16_t LINE_BUFFER_SIZE = (16 * 2) + 16 + 1;  // Show 16 bytes, 2 chars/byte, spaces between bytes, null termination
+#endif //defined(_MSC_VER)
+
 #ifdef __ANDROID__
-#elif defined __linux__ || defined __APPLE__
+#elif defined __linux__ || defined __APPLE__ || defined _WIN32
 static oc_log_level LEVEL_XTABLE[] = {OC_LOG_DEBUG, OC_LOG_INFO,
                                       OC_LOG_WARNING, OC_LOG_ERROR, OC_LOG_FATAL};
 #endif
 
-// Show 16 bytes, 2 chars/byte, spaces between bytes, null termination
-static const uint16_t LINE_BUFFER_SIZE = (16 * 2) + 16 + 1;
-
 // Convert LogLevel to platform-specific severity level.  Store in PROGMEM on Arduino
 #ifdef __ANDROID__
 #ifdef ADB_SHELL
@@ -72,9 +81,10 @@ static const uint16_t LINE_BUFFER_SIZE = (16 * 2) + 16 + 1;
     static android_LogPriority LEVEL[] =
     {ANDROID_LOG_DEBUG, ANDROID_LOG_INFO, ANDROID_LOG_WARN, ANDROID_LOG_ERROR, ANDROID_LOG_FATAL};
 #endif
-#elif defined (__linux__) || defined (__APPLE__)
-    static const char *LEVEL[] __attribute__ ((unused)) =
-    {"DEBUG", "INFO", "WARNING", "ERROR", "FATAL"};
+#elif defined(__linux__) || defined(__APPLE__) || defined(__msys_nt__)
+    static const char * LEVEL[] __attribute__ ((unused)) = {"DEBUG", "INFO", "WARNING", "ERROR", "FATAL"};
+#elif defined(_MSC_VER)
+    static const char * LEVEL[] = {"DEBUG", "INFO", "WARNING", "ERROR", "FATAL"};
 #elif defined ARDUINO
 #include <stdarg.h>
 #include "Arduino.h"
@@ -98,111 +108,12 @@ static const uint16_t LINE_BUFFER_SIZE = (16 * 2) + 16 + 1;
 #else
     #define GET_PROGMEM_BUFFER(buffer, addr) { buffer[0] = '\0';}
 #endif
-#endif // __ANDROID__
-
+#else // !defined(__ANDROID__) && !defined(ARDUINO)
+    static const char *LEVEL[] __attribute__ ((unused)) =
+    {"DEBUG", "INFO", "WARNING", "ERROR", "FATAL"};
+#endif
 
 #ifndef ARDUINO
-#ifndef __TIZEN__
-void OCLogConfig(oc_log_ctx_t *ctx)
-{
-    logCtx = ctx;
-}
-
-void OCLogInit()
-{
-
-}
-
-void OCLogShutdown()
-{
-#if defined(__linux__) || defined(__APPLE__)
-    if (logCtx && logCtx->destroy)
-    {
-        logCtx->destroy(logCtx);
-    }
-#endif
-}
-
-/**
- * Output a variable argument list log string with the specified priority level.
- * Only defined for Linux and Android
- *
- * @param level  - DEBUG, INFO, WARNING, ERROR, FATAL
- * @param tag    - Module name
- * @param format - variadic log string
- */
-void OCLogv(LogLevel level, const char * tag, const char * format, ...)
-{
-    if (!format || !tag) {
-        return;
-    }
-    char buffer[MAX_LOG_V_BUFFER_SIZE] = {};
-    va_list args;
-    va_start(args, format);
-    vsnprintf(buffer, sizeof buffer - 1, format, args);
-    va_end(args);
-    OCLog(level, tag, buffer);
-}
-
-/**
- * Output a log string with the specified priority level.
- * Only defined for Linux and Android
- *
- * @param level  - DEBUG, INFO, WARNING, ERROR, FATAL
- * @param tag    - Module name
- * @param logStr - log string
- */
-void OCLog(LogLevel level, const char * tag, const char * logStr)
-{
-    if (!logStr || !tag)
-    {
-       return;
-    }
-
-   #ifdef __ANDROID__
-
-   #ifdef ADB_SHELL
-       printf("%s: %s: %s\n", LEVEL[level], tag, logStr);
-   #else
-       __android_log_write(LEVEL[level], tag, logStr);
-   #endif
-
-   #elif defined __linux__ || defined __APPLE__
-       if (logCtx && logCtx->write_level)
-       {
-           logCtx->write_level(logCtx, LEVEL_XTABLE[level], logStr);
-
-       }
-       else
-       {
-           int min = 0;
-           int sec = 0;
-           int ms = 0;
-   #if defined(_POSIX_TIMERS) && _POSIX_TIMERS > 0
-           struct timespec when = { .tv_sec = 0, .tv_nsec = 0 };
-           clockid_t clk = CLOCK_REALTIME;
-   #ifdef CLOCK_REALTIME_COARSE
-           clk = CLOCK_REALTIME_COARSE;
-   #endif
-           if (!clock_gettime(clk, &when))
-           {
-               min = (when.tv_sec / 60) % 60;
-               sec = when.tv_sec % 60;
-               ms = when.tv_nsec / 1000000;
-           }
-   #else
-           struct timeval now;
-           if (!gettimeofday(&now, NULL))
-           {
-               min = (now.tv_sec / 60) % 60;
-               sec = now.tv_sec % 60;
-               ms = now.tv_usec * 1000;
-           }
-   #endif
-           printf("%02d:%02d.%03d %s: %s: %s\n", min, sec, ms, LEVEL[level], tag, logStr);
-       }
-   #endif
-   }
 
 /**
  * Output the contents of the specified buffer (in hex) with the specified priority level.
@@ -233,7 +144,7 @@ void OCLogBuffer(LogLevel level, const char * tag, const uint8_t * buffer, uint1
         // Output 16 values per line
         if (((i+1)%16) == 0)
         {
-            OCLog(level, tag, lineBuffer);
+            OCLogv(level, tag, "%s", lineBuffer);
             memset(lineBuffer, 0, sizeof lineBuffer);
             lineIndex = 0;
         }
@@ -241,9 +152,116 @@ void OCLogBuffer(LogLevel level, const char * tag, const uint8_t * buffer, uint1
     // Output last values in the line, if any
     if (bufferSize % 16)
     {
-        OCLog(level, tag, lineBuffer);
+        OCLogv(level, tag, "%s", lineBuffer);
     }
 }
+#ifndef __TIZEN__
+void OCLogConfig(oc_log_ctx_t *ctx)
+{
+    logCtx = ctx;
+}
+
+void OCLogInit()
+{
+
+}
+
+void OCLogShutdown()
+{
+#if defined(__linux__) || defined(__APPLE__) || defined(_WIN32)
+    if (logCtx && logCtx->destroy)
+    {
+        logCtx->destroy(logCtx);
+    }
+#endif
+}
+
+/**
+ * Output a variable argument list log string with the specified priority level.
+ * Only defined for Linux and Android
+ *
+ * @param level  - DEBUG, INFO, WARNING, ERROR, FATAL
+ * @param tag    - Module name
+ * @param format - variadic log string
+ */
+void OCLogv(LogLevel level, const char * tag, const char * format, ...)
+{
+    if (!format || !tag) {
+        return;
+    }
+    char buffer[MAX_LOG_V_BUFFER_SIZE] = {0};
+    va_list args;
+    va_start(args, format);
+    vsnprintf(buffer, sizeof buffer - 1, format, args);
+    va_end(args);
+    OCLog(level, tag, buffer);
+}
+
+/**
+ * Output a log string with the specified priority level.
+ * Only defined for Linux and Android
+ *
+ * @param level  - DEBUG, INFO, WARNING, ERROR, FATAL
+ * @param tag    - Module name
+ * @param logStr - log string
+ */
+void OCLog(LogLevel level, const char * tag, const char * logStr)
+{
+    if (!logStr || !tag)
+    {
+       return;
+    }
+
+   #ifdef __ANDROID__
+
+   #ifdef ADB_SHELL
+       printf("%s: %s: %s\n", LEVEL[level], tag, logStr);
+   #else
+       __android_log_write(LEVEL[level], tag, logStr);
+   #endif
+
+   #else
+       if (logCtx && logCtx->write_level)
+       {
+           logCtx->write_level(logCtx, LEVEL_XTABLE[level], logStr);
+
+       }
+       else
+       {
+           int min = 0;
+           int sec = 0;
+           int ms = 0;
+   #if defined(_POSIX_TIMERS) && _POSIX_TIMERS > 0
+           struct timespec when = { .tv_sec = 0, .tv_nsec = 0 };
+           clockid_t clk = CLOCK_REALTIME;
+   #ifdef CLOCK_REALTIME_COARSE
+           clk = CLOCK_REALTIME_COARSE;
+   #endif
+           if (!clock_gettime(clk, &when))
+           {
+               min = (when.tv_sec / 60) % 60;
+               sec = when.tv_sec % 60;
+               ms = when.tv_nsec / 1000000;
+           }
+   #elif defined(_WIN32)
+           SYSTEMTIME systemTime = {0};
+           GetLocalTime(&systemTime);
+           min = (int)systemTime.wMinute;
+           sec = (int)systemTime.wSecond;
+           ms  = (int)systemTime.wMilliseconds;
+   #else
+           struct timeval now;
+           if (!gettimeofday(&now, NULL))
+           {
+               min = (now.tv_sec / 60) % 60;
+               sec = now.tv_sec % 60;
+               ms = now.tv_usec * 1000;
+           }
+   #endif
+           printf("%02d:%02d.%03d %s: %s: %s\n", min, sec, ms, LEVEL[level], tag, logStr);
+       }
+   #endif
+   }
 #endif //__TIZEN__
 #endif //ARDUINO
 #ifdef ARDUINO
