@@ -22,6 +22,9 @@
 #define IOTVT_SRM_DOXM_H
 
 #include "octypes.h"
+#ifdef _ENABLE_MULTIPLE_OWNER_
+#include "cacommon.h"
+#endif //_ENABLE_MULTIPLE_OWNER_
 
 #ifdef __cplusplus
 extern "C" {
@@ -72,11 +75,12 @@ OCStackResult CBORPayloadToDoxm(const uint8_t *cborPayload, size_t size,
  * @note Caller needs to invoke OCFree after done using the return pointer.
  * @param cborPayload is the payload of the cbor.
  * @param cborSize is the size of the cbor payload. Passed parameter should not be NULL.
+ * @param rwOnly indicates whether convertingpayload has all properties or read-write properties only.
  *
  * @return ::OC_STACK_OK for Success, otherwise some error value.
  */
 OCStackResult DoxmToCBORPayload(const OicSecDoxm_t * doxm, uint8_t **cborPayload,
-                                size_t *cborSize);
+                                size_t *cborSize, bool rwOnly);
 
 /**
  * This method returns the SRM device ID for this device.
@@ -85,6 +89,14 @@ OCStackResult DoxmToCBORPayload(const OicSecDoxm_t * doxm, uint8_t **cborPayload
  */
 OCStackResult GetDoxmDeviceID(OicUuid_t *deviceID);
 
+/**
+ * This method changes the SRM device ID for this device.
+ * This api will update device Id iff device is in unowned state.
+ * @return ::OC_STACK_OK for Success, otherwise some error value.
+ */
+OCStackResult SetDoxmDeviceID(const OicUuid_t *deviceID);
+
+
 /**
  * Gets the OicUuid_t value for the owner of this device.
  *
@@ -93,6 +105,14 @@ OCStackResult GetDoxmDeviceID(OicUuid_t *deviceID);
  */
 OCStackResult GetDoxmDevOwnerId(OicUuid_t *devownerid);
 
+/**
+ * Gets the bool state of "isOwned" property on the doxm resource.
+ *
+ * @param isOwned a pointer to be assigned to isOwned property
+ * @return ::OC_STACK_OK if isOwned is assigned correctly, else ::OC_STACK_ERROR.
+ */
+OCStackResult GetDoxmIsOwned(bool *isOwned);
+
 /**
  * Gets the OicUuid_t value for the rowneruuid of the doxm resource.
  *
@@ -101,6 +121,17 @@ OCStackResult GetDoxmDevOwnerId(OicUuid_t *devownerid);
  */
 OCStackResult GetDoxmRownerId(OicUuid_t *rowneruuid);
 
+#ifdef _ENABLE_MULTIPLE_OWNER_
+/**
+ * Compare the UUID to SubOwner.
+ *
+ * @param[in] uuid device UUID
+ *
+ * @return true if uuid exists in the SubOwner list of doxm, else false.
+ */
+bool IsSubOwner(const OicUuid_t* uuid);
+#endif //_ENABLE_MULTIPLE_OWNER_
+
 /** This function deallocates the memory for OicSecDoxm_t .
  *
  * @param doxm is the pointer to @ref OicSecDoxm_t.
@@ -113,6 +144,16 @@ void DeleteDoxmBinData(OicSecDoxm_t* doxm);
  */
 void RestoreDoxmToInitState();
 
+#if defined(__WITH_DTLS__) && defined(_ENABLE_MULTIPLE_OWNER_)
+/**
+ * Callback function to handle MOT DTLS handshake result.
+ * @param[out]   object           remote device information.
+ * @param[out]   errorInfo        CA Error information.
+ */
+void MultipleOwnerDTLSHandshakeCB(const CAEndpoint_t *object,
+                                const CAErrorInfo_t *errorInfo);
+#endif //__WITH_DTLS__ && _ENABLE_MULTIPLE_OWNER_
+
 #ifdef __cplusplus
 }
 #endif
