@@ -23,9 +23,12 @@
 
 #include "securevirtualresourcetypes.h"
 
+#ifdef __cplusplus
+extern "C" {
+#endif
+
 extern const char * SVR_DB_FILE_NAME;
 extern const char * SVR_DB_DAT_FILE_NAME;
-extern const char * OIC_MI_DEF;
 
 //AMACL
 extern const char * OIC_RSRC_TYPE_SEC_AMACL;
@@ -86,6 +89,9 @@ extern const char * OIC_RSRC_TYPE_SEC_VER;
 extern const char * OIC_RSRC_VER_URI;
 extern const char * OIC_JSON_VER_NAME;
 
+//reset profile
+extern const char * OIC_JSON_RESET_PF_NAME;
+
 extern const char * OIC_JSON_SUBJECT_NAME;
 extern const char * OIC_JSON_SUBJECTID_NAME;
 extern const char * OIC_JSON_RESOURCES_NAME;
@@ -95,9 +101,15 @@ extern const char * OIC_JSON_PERMISSION_NAME;
 extern const char * OIC_JSON_OWNERS_NAME;
 extern const char * OIC_JSON_OWNER_NAME;
 extern const char * OIC_JSON_DEVOWNERID_NAME;
+#ifdef _ENABLE_MULTIPLE_OWNER_
+extern const char * OIC_JSON_SUBOWNERID_NAME;
+#endif //_ENABLE_MULTIPLE_OWNER_
 extern const char * OIC_JSON_OWNED_NAME;
 extern const char * OIC_JSON_OXM_NAME;
 extern const char * OIC_JSON_OXMS_NAME;
+#ifdef _ENABLE_MULTIPLE_OWNER_
+extern const char * OIC_JSON_MOM_NAME;
+#endif //_ENABLE_MULTIPLE_OWNER_
 extern const char * OIC_JSON_OXM_TYPE_NAME;
 extern const char * OIC_JSON_OXM_SEL_NAME;
 extern const char * OIC_JSON_DEVICE_ID_FORMAT_NAME;
@@ -109,7 +121,9 @@ extern const char * OIC_JSON_PRIVATEDATA_NAME;
 extern const char * OIC_JSON_PUBDATA_NAME;
 extern const char * OIC_JSON_PRIVDATA_NAME;
 extern const char * OIC_JSON_OPTDATA_NAME;
+extern const char * OIC_JSON_CREDUSAGE_NAME;
 extern const char * OIC_JSON_CRMS_NAME;
+extern const char * OIC_JSON_VALIDITY_NAME;
 extern const char * OIC_JSON_PERIOD_NAME;
 extern const char * OIC_JSON_PERIODS_NAME;
 extern const char * OIC_JSON_RECURRENCES_NAME;
@@ -138,12 +152,23 @@ extern const char * OIC_JSON_REL_NAME;
 extern const char * OIC_JSON_RT_NAME;
 extern const char * OIC_JSON_IF_NAME;
 extern const char * OIC_JSON_ROWNERID_NAME;
+#ifdef _ENABLE_MULTIPLE_OWNER_
+extern const char * OIC_JSON_EOWNERID_NAME;
+#endif //_ENABLE_MULTIPLE_OWNER_
 extern const char * OIC_JSON_ENCODING_NAME;
 extern const char * OIC_JSON_DATA_NAME;
 extern const char * OIC_JSON_SEC_V_NAME;
 
 extern const char * OIC_JSON_EMPTY_STRING;
 
+// Certificates provided by Cloud
+extern const char * TRUST_CA;
+extern const char * PRIMARY_CERT;
+
+// Certificates provided by manufacturer
+extern const char * MF_TRUST_CA;
+extern const char * MF_PRIMARY_CERT;
+
 extern OicUuid_t WILDCARD_SUBJECT_ID;
 extern OicUuid_t WILDCARD_SUBJECT_B64_ID;
 extern size_t WILDCARD_SUBJECT_ID_LEN;
@@ -153,9 +178,14 @@ extern const char * WILDCARD_RESOURCE_URI;
 extern const char * OXM_JUST_WORKS;
 extern const char * OXM_RANDOM_DEVICE_PIN;
 extern const char * OXM_MANUFACTURER_CERTIFICATE;
+#ifdef _ENABLE_MULTIPLE_OWNER_
+extern const char * OXM_PRECONF_PIN;
+#endif //_ENABLE_MULTIPLE_OWNER_
 
 extern const char * OIC_SEC_ENCODING_BASE64;
 extern const char * OIC_SEC_ENCODING_RAW;
+extern const char * OIC_SEC_ENCODING_PEM;
+extern const char * OIC_SEC_ENCODING_DER;
 
 extern const char * OIC_SEC_TRUE;
 extern const char * OIC_SEC_FALSE;
@@ -166,5 +196,9 @@ extern char OIC_SEC_REST_QUERY_DELIMETER;
 //Security Version
 extern const char * DEFAULT_SEC_VERSION;
 
+#ifdef __cplusplus
+}
+#endif
+
 #endif //IOTVT_SRM_RSRC_STRINGS_H
 
