@@ -20,17 +20,31 @@
 ///////////////////////////////////////////////////////////////////////
 //NOTE :  This sample server is generated based on ocserverbasicops.cpp
 ///////////////////////////////////////////////////////////////////////
+#include "iotivity_config.h"
 #include <stdio.h>
 #include <string.h>
 #include <stdlib.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#ifdef HAVE_TIME_H
+#include <time.h>
+#endif
 #include <signal.h>
-#include <pthread.h>
 #include "ocstack.h"
-#include "logger.h"
 #include "ocpayload.h"
 #include "pinoxmcommon.h"
 
+#ifdef HAVE_WINDOWS_H
+#include <windows.h>
+/** @todo stop-gap for naming issue. Windows.h does not like us to use ERROR */
+#ifdef ERROR
+#undef ERROR
+#endif //ERROR
+#endif //HAVE_WINDOWS_H
+#include "platform_features.h"
+#include "logger.h"
+
 #define TAG "SAMPLE_RANDOMPIN"
 
 int gQuitFlag = 0;
@@ -128,6 +142,43 @@ const char *getResult(OCStackResult result) {
     }
 }
 
+#ifdef _ENABLE_MULTIPLE_OWNER_
+
+#include <assert.h>
+#include <thread>
+#include <chrono>
+
+static bool volatile g_LoopFlag;
+static std::thread* oc_process_thread;
+
+static void oc_process_loop()
+{
+    while(g_LoopFlag)
+    {
+        if (OCProcess() != OC_STACK_OK)
+        {
+            OIC_LOG(ERROR, TAG, "OCStack process error");
+            g_LoopFlag = false;
+            break;
+        }
+        std::this_thread::sleep_for(std::chrono::milliseconds(100));
+    }
+}
+
+static void StartOCProcessThread()
+{
+    g_LoopFlag = true;
+    oc_process_thread = new std::thread(oc_process_loop);
+}
+
+static void StopOCProcessThread()
+{
+    assert(oc_process_thread->joinable() == true);
+    g_LoopFlag = false;
+    oc_process_thread->join();
+}
+#endif //_ENABLE_MULTIPLE_OWNER_
+
 OCRepPayload* getPayload(const char* uri, int64_t power, bool state)
 {
     OCRepPayload* payload = OCRepPayloadCreate();
@@ -263,7 +314,7 @@ OCEntityHandlerResult ProcessPostRequest (OCEntityHandlerRequest *ehRequest,
                 gLedInstance[gCurrLedInstance].state = 0;
                 gLedInstance[gCurrLedInstance].power = 0;
                 gCurrLedInstance++;
-                strncpy ((char *)response->resourceUri, newLedUri, MAX_URI_LENGTH);
+                strncpy ((char *)response->resourceUri, newLedUri, sizeof(response->resourceUri));
                 ehResult = OC_EH_RESOURCE_CREATED;
             }
         }
@@ -411,8 +462,6 @@ void GeneratePinCB(char* pin, size_t pinSize)
 
 int main()
 {
-    struct timespec timeout;
-
     OIC_LOG(DEBUG, TAG, "OCServer is starting...");
 
     // Initialize Persistent Storage for SVR database
@@ -429,19 +478,53 @@ int main()
      * If server supported random pin based ownership transfer,
      * callback of print PIN should be registered before runing server.
      */
-    SetGeneratePinCB(&GeneratePinCB);
+    SetGeneratePinCB(GeneratePinCB);
+
+    /**
+     * Random PIN generation policy can be changed through SetRandomPinPolicy() API.
+     * first param : byte length of random PIN ( 4 <= first param <= 32)
+     * second param : PIN type (This is bitmask)
+     */
+    if(OC_STACK_OK != SetRandomPinPolicy(8, NUM_PIN))
+    {
+        OIC_LOG(ERROR, TAG, "Failed to setting PIN policy");
+        return 0;
+    }
 
     /*
      * Declare and create the example resource: LED
      */
     createLEDResource(gResourceUri, &LED, false, 0);
 
-    timeout.tv_sec  = 0;
-    timeout.tv_nsec = 100000000L;
-
     // Break from loop with Ctrl-C
     OIC_LOG(INFO, TAG, "Entering ocserver main loop...");
     signal(SIGINT, handleSigInt);
+
+#ifdef _ENABLE_MULTIPLE_OWNER_
+    StartOCProcessThread();
+
+    while(!gQuitFlag)
+    {
+        printf("Press 'G' to generate random PIN...\n");
+        printf("Press 'E' to exit...\n");
+        char in = getchar();
+        if('G' == in || 'g' == in)
+        {
+            char ranPin[OXM_RANDOM_PIN_MAX_SIZE + 1] = {0};
+            GeneratePin(ranPin, sizeof(ranPin));
+        }
+        if('E' == in || 'e' == in)
+        {
+            break;
+        }
+    }
+
+    StopOCProcessThread();
+#else
+    struct timespec timeout;
+    timeout.tv_sec  = 0;
+    timeout.tv_nsec = 100000000L;
+
     while (!gQuitFlag)
     {
         if (OCProcess() != OC_STACK_OK)
@@ -451,6 +534,7 @@ int main()
         }
         nanosleep(&timeout, NULL);
     }
+#endif //_ENABLE_MULTIPLE_OWNER_
 
     OIC_LOG(INFO, TAG, "Exiting ocserver main loop...");
 
