@@ -17,6 +17,7 @@
  * limitations under the License.
  *
  * *****************************************************************/
+#include "iotivity_config.h"
 #include "gtest/gtest.h"
 #include "provisioningdatabasemanager.h"
 
@@ -74,10 +75,12 @@ TEST(PDMIsDuplicateDeviceTest, ValidUUID)
 {
     OicUuid_t uid1 = {{0,}};
     memcpy(&uid1.id, ID_1, sizeof(uid1.id));
-    PDMAddDevice(&uid1);
+    EXPECT_EQ(OC_STACK_OK, PDMAddDevice(&uid1));
+    EXPECT_EQ(OC_STACK_OK, PDMSetDeviceState(&uid1, PDM_DEVICE_ACTIVE));
     OicUuid_t uid2 = {{0,}};
     memcpy(&uid2.id, ID_2, sizeof(uid2.id));
-    PDMAddDevice(&uid2);
+    EXPECT_EQ(OC_STACK_OK, PDMAddDevice(&uid2));
+    EXPECT_EQ(OC_STACK_OK, PDMSetDeviceState(&uid2, PDM_DEVICE_ACTIVE));
     OicUuid_t uid3 = {{0,}};
     memcpy(&uid3.id, ID_3, sizeof(uid3.id));
     bool isDuplicate = true;
@@ -103,7 +106,7 @@ TEST(PDMAddDeviceTest, ValidUUID)
     memcpy(&uid.id, &id, UUID_LENGTH);
 
     EXPECT_EQ(OC_STACK_OK, PDMAddDevice(&uid));
-    PDMDeleteDevice(&uid);
+    EXPECT_EQ(OC_STACK_OK, PDMDeleteDevice(&uid));
 }
 
 TEST(PDMLinkDevicesTest, NULLDevice1)
@@ -124,10 +127,14 @@ TEST(PDMLinkDevicesTest, ValidCase)
 {
     OicUuid_t uid1 = {{0,}};
     memcpy(&uid1.id, ID_4, sizeof(uid1.id));
-    PDMAddDevice(&uid1);
+    EXPECT_EQ(OC_STACK_OK, PDMAddDevice(&uid1));
+    EXPECT_EQ(OC_STACK_OK, PDMSetDeviceState(&uid1, PDM_DEVICE_ACTIVE));
+
     OicUuid_t uid2 = {{0,}};
     memcpy(&uid2.id, ID_5, sizeof(uid2.id));
-    PDMAddDevice(&uid2);
+    EXPECT_EQ(OC_STACK_OK, PDMAddDevice(&uid2));
+    EXPECT_EQ(OC_STACK_OK, PDMSetDeviceState(&uid2, PDM_DEVICE_ACTIVE));
+
     EXPECT_EQ(OC_STACK_OK, PDMLinkDevices(&uid1, &uid2));
 }
 
@@ -233,6 +240,8 @@ TEST(PDMSetLinkStale, ValidCase)
 
     EXPECT_EQ(OC_STACK_OK, PDMAddDevice(&uid1));
 
+    EXPECT_EQ(OC_STACK_OK, PDMSetDeviceState(&uid1, PDM_DEVICE_ACTIVE));
+
     EXPECT_EQ(OC_STACK_OK, PDMLinkDevices(&uid1, &uid2));
 
     EXPECT_EQ(OC_STACK_OK, PDMSetLinkStale(&uid1, &uid2));
@@ -266,9 +275,12 @@ TEST(PDMIsLinkExistsTest, DuplicateID)
     OicUuid_t uid1 = {{0,}};
     memcpy(&uid1.id, ID_7, sizeof(uid1.id));
     EXPECT_EQ(OC_STACK_OK, PDMAddDevice(&uid1));
+    EXPECT_EQ(OC_STACK_OK, PDMSetDeviceState(&uid1, PDM_DEVICE_ACTIVE));
+
     OicUuid_t uid2 = {{0,}};
     memcpy(&uid2.id, ID_8, sizeof(uid2.id));
     EXPECT_EQ(OC_STACK_OK, PDMAddDevice(&uid2));
+    EXPECT_EQ(OC_STACK_OK, PDMSetDeviceState(&uid2, PDM_DEVICE_ACTIVE));
 
     bool linkExisits = true;
     OCStackResult res = PDMIsLinkExists(&uid1, &uid2, &linkExisits);
@@ -279,7 +291,7 @@ TEST(PDMIsLinkExistsTest, DuplicateID)
 TEST(PDMSetDeviceStaleTest, NULLUUID)
 {
     EXPECT_EQ(OC_STACK_OK, PDMInit(NULL));
-    EXPECT_EQ(OC_STACK_INVALID_PARAM, PDMSetDeviceStale(NULL));
+    EXPECT_EQ(OC_STACK_INVALID_PARAM, PDMSetDeviceState(NULL, PDM_DEVICE_STALE));
 }
 
 TEST(PDMSetDeviceStaleTest, VALIDUUID)
@@ -288,7 +300,7 @@ TEST(PDMSetDeviceStaleTest, VALIDUUID)
     OicUuid_t uid1 = {{0,}};
     memcpy(&uid1.id, ID_9, sizeof(uid1.id));
     EXPECT_EQ(OC_STACK_OK, PDMAddDevice(&uid1));
-    EXPECT_EQ(OC_STACK_OK,PDMSetDeviceStale(&uid1));
+    EXPECT_EQ(OC_STACK_OK,PDMSetDeviceState(&uid1, PDM_DEVICE_STALE));
 }
 
 TEST(PDMSetDeviceStaleTest, StaleDeviceNotinDeviceList)
@@ -297,7 +309,7 @@ TEST(PDMSetDeviceStaleTest, StaleDeviceNotinDeviceList)
     OicUuid_t uid1 = {{0,}};
     memcpy(&uid1.id, ID_10, sizeof(uid1.id));
     EXPECT_EQ(OC_STACK_OK, PDMAddDevice(&uid1));
-    EXPECT_EQ(OC_STACK_OK,PDMSetDeviceStale(&uid1));
+    EXPECT_EQ(OC_STACK_OK, PDMSetDeviceState(&uid1, PDM_DEVICE_STALE));
 
     OCUuidList_t *list = NULL;
     size_t noOfDevcies = 0;
@@ -323,14 +335,17 @@ TEST(PDMSetDeviceStaleTest, StaleDeviceNotinLinkedDevice)
     memcpy(&uid3.id, ID_13, sizeof(uid3.id));
 
     EXPECT_EQ(OC_STACK_OK, PDMAddDevice(&uid1));
+    EXPECT_EQ(OC_STACK_OK, PDMSetDeviceState(&uid1, PDM_DEVICE_ACTIVE));
     EXPECT_EQ(OC_STACK_OK, PDMAddDevice(&uid2));
+    EXPECT_EQ(OC_STACK_OK, PDMSetDeviceState(&uid2, PDM_DEVICE_ACTIVE));
     EXPECT_EQ(OC_STACK_OK, PDMAddDevice(&uid3));
+    EXPECT_EQ(OC_STACK_OK, PDMSetDeviceState(&uid3, PDM_DEVICE_ACTIVE));
 
     EXPECT_EQ(OC_STACK_OK, PDMLinkDevices(&uid1, &uid2));
     EXPECT_EQ(OC_STACK_OK, PDMLinkDevices(&uid2, &uid3));
     EXPECT_EQ(OC_STACK_OK, PDMLinkDevices(&uid1, &uid3));
 
-    EXPECT_EQ(OC_STACK_OK,PDMSetDeviceStale(&uid1));
+    EXPECT_EQ(OC_STACK_OK,PDMSetDeviceState(&uid1, PDM_DEVICE_STALE));
 
     OCUuidList_t *list1 = NULL;
     size_t noOfDevices1 = 0;
