@@ -20,12 +20,19 @@
 #ifndef _POSIX_C_SOURCE
 #define _POSIX_C_SOURCE 200112L
 #endif
-#ifndef WITH_ARDUINO
+#include "iotivity_config.h"
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#ifdef HAVE_TIME_H
 #include <time.h>
+#endif
+#ifdef HAVE_SYS_TIME_H
 #include <sys/time.h>
 #endif
+#ifdef HAVE_STRING_H
 #include <string.h>
+#endif
 
 #include "ocstack.h"
 #include "oic_malloc.h"
@@ -54,7 +61,7 @@
 #endif
 
 
-#define TAG ("DP")
+#define TAG ("OIC_DP")
 static const uint16_t CBOR_SIZE = 1024;
 
 /**
@@ -65,6 +72,7 @@ typedef struct DPairData
     OCDirectPairingDev_t        *peer;                         /**< Pointer to pairing target info.**/
     char                                  pin[DP_PIN_LENGTH];  /**< PIN **/
     OCDirectPairingResultCB    resultCallback;           /**< Pointer to result callback.**/
+    void *userCtx;                                      /** < user context to pass in callback **/
 } DPairData_t;
 
 static OCDirectPairingDev_t *g_dp_paired = NULL;
@@ -238,23 +246,46 @@ bool DPGenerateQuery(bool isSecure,
 
     static char QPREFIX_COAP[] =  "coap://";
     static char QPREFIX_COAPS[] = "coaps://";
+    static char QPREFIX_COAP_TCP[] =  "coap+tcp://";
+    static char QPREFIX_COAPS_TCP[] = "coaps+tcp://";
 
     int snRet = 0;
     char* prefix = (isSecure == true) ? QPREFIX_COAPS : QPREFIX_COAP;
 
     switch(connType & CT_MASK_ADAPTER)
     {
+// @todo: Remove this ifdef. On Arduino, CT_ADAPTER_TCP resolves to the same value
+// as CT_ADAPTER_IP, resulting in a compiler error.
+#ifdef WITH_TCP
+#ifndef WITH_ARDUINO
+        case CT_ADAPTER_TCP:
+            prefix = (isSecure == true) ? QPREFIX_COAPS_TCP : QPREFIX_COAP_TCP;
+#endif
+#endif
         case CT_ADAPTER_IP:
             switch(connType & CT_MASK_FLAGS & ~CT_FLAG_SECURE)
             {
                 case CT_IP_USE_V4:
-                        snRet = snprintf(buffer, bufferSize, "%s%s:%d%s",
-                                         prefix, address, port, uri);
+                    snRet = snprintf(buffer, bufferSize, "%s%s:%d%s",
+                                     prefix, address, port, uri);
                     break;
                 case CT_IP_USE_V6:
-                        snRet = snprintf(buffer, bufferSize, "%s[%s]:%d%s",
-                                         prefix, address, port, uri);
+                {
+                    char addressEncoded[CA_MAX_URI_LENGTH] = {0};
+
+                    OCStackResult result = OCEncodeAddressForRFC6874(addressEncoded,
+                                                                     sizeof(addressEncoded),
+                                                                     address);
+                    if (OC_STACK_OK != result)
+                    {
+                        OIC_LOG_V(ERROR, TAG, "DPGenerateQuery : encoding error %d\n", result);
+                        return false;
+                    }
+
+                    snRet = snprintf(buffer, bufferSize, "%s[%s]:%d%s",
+                                     prefix, addressEncoded, port, uri);
                     break;
+                }
                 default:
                     OIC_LOG(ERROR, TAG, "Unknown address format.");
                     return false;
@@ -331,7 +362,7 @@ static OCStackApplicationResult DirectPairingFinalizeHandler(void *ctx, OCDoHand
 
     if (clientResponse)
     {
-        if(OC_STACK_OK == clientResponse->result)
+        if(OC_STACK_RESOURCE_CHANGED == clientResponse->result)
         {
             // result
             OIC_LOG(INFO, TAG, "DirectPairingFinalizeHandler : success PUT"
@@ -347,32 +378,32 @@ static OCStackApplicationResult DirectPairingFinalizeHandler(void *ctx, OCDoHand
             if (OC_STACK_OK != GetDoxmDeviceID(&ptDeviceID))
             {
                 OIC_LOG(ERROR, TAG, "Error while retrieving provisioning tool's device ID");
-                resultCallback(peer, OC_STACK_ERROR);
+                resultCallback(dpairData->userCtx, peer, OC_STACK_ERROR);
                 return OC_STACK_DELETE_TRANSACTION;
             }
 
-#ifdef __WITH_DTLS__
+#if defined(__WITH_DTLS__) || defined(__WITH_TLS__)
             res = SavePairingPSK((OCDevAddr*)&endpoint, &peer->deviceID, &ptDeviceID, false);
             if(OC_STACK_OK != res)
             {
                 OIC_LOG(ERROR, TAG, "Failed to PairingPSK generation");
-                resultCallback(peer, res);
+                resultCallback(dpairData->userCtx, peer, res);
                 return OC_STACK_DELETE_TRANSACTION;
             }
 
             //  close temporary sesion
-            CAResult_t caResult = CACloseDtlsSession((const CAEndpoint_t*)&endpoint);
+            CAResult_t caResult = CAcloseSslSession((const CAEndpoint_t*)&endpoint);
             if(CA_STATUS_OK != caResult)
             {
                 OIC_LOG(INFO, TAG, "Fail to close temporary dtls session");
             }
 
-            caResult = CASelectCipherSuite(TLS_NULL_WITH_NULL_NULL);
+            caResult = CASelectCipherSuite(TLS_NULL_WITH_NULL_NULL, CA_ADAPTER_IP);
             if(CA_STATUS_OK != caResult)
             {
                 OIC_LOG(ERROR, TAG, "Failed to select TLS_NULL_WITH_NULL_NULL");
             }
-#endif // __WITH_DTLS__
+#endif // __WITH_DTLS__ or __WITH_TLS__
 
             OIC_LOG(INFO, TAG, "Direct-Papring was successfully completed.");
 
@@ -385,7 +416,7 @@ static OCStackApplicationResult DirectPairingFinalizeHandler(void *ctx, OCDoHand
                 OIC_LOG(ERROR, TAG, "Error while adding a device to paired list.");
             }
 
-            resultCallback(peer, OC_STACK_OK);
+            resultCallback(dpairData->userCtx, peer, OC_STACK_OK);
 
             return OC_STACK_DELETE_TRANSACTION;
         }
@@ -399,7 +430,7 @@ static OCStackApplicationResult DirectPairingFinalizeHandler(void *ctx, OCDoHand
         OIC_LOG(ERROR, TAG, "DirectPairingFinalizeHandler received Null clientResponse");
     }
 
-    resultCallback(peer, OC_STACK_ERROR);
+    resultCallback(dpairData->userCtx, peer, OC_STACK_ERROR);
     OICFree(dpairData);
     return OC_STACK_DELETE_TRANSACTION;
 }
@@ -412,7 +443,7 @@ static OCStackApplicationResult DirectPairingFinalizeHandler(void *ctx, OCDoHand
  *
  * @return OC_STACK_OK on success otherwise error.
  */
-OCStackResult FinalizeDirectPairing(OCDirectPairingDev_t* peer,
+OCStackResult FinalizeDirectPairing(void *ctx, OCDirectPairingDev_t* peer,
                                                      OCDirectPairingResultCB resultCallback)
 {
     if(NULL == peer)
@@ -449,8 +480,8 @@ OCStackResult FinalizeDirectPairing(OCDirectPairingDev_t* peer,
         OIC_LOG(ERROR, TAG, "Failed to DpairingToCBORPayload");
         return OC_STACK_NO_MEMORY;
     }
-    OIC_LOG(INFO, TAG, "DPARING CBOR data:");
-    OIC_LOG_BUFFER(INFO, TAG, secPayload->securityData, secPayload->payloadSize);
+    OIC_LOG(DEBUG, TAG, "DPARING CBOR data:");
+    OIC_LOG_BUFFER(DEBUG, TAG, secPayload->securityData, secPayload->payloadSize);
 
     char query[MAX_URI_LENGTH + MAX_QUERY_LENGTH] = {0};
     if(!DPGenerateQuery(true,
@@ -474,6 +505,7 @@ OCStackResult FinalizeDirectPairing(OCDirectPairingDev_t* peer,
     }
     dpairData->peer = peer;
     dpairData->resultCallback = resultCallback;
+    dpairData->userCtx = ctx;
 
     OCCallbackData cbData =  {.context=NULL, .cb=NULL, .cd=NULL};
     cbData.cb = DirectPairingFinalizeHandler;
@@ -525,22 +557,22 @@ void DirectPairingDTLSHandshakeCB(const CAEndpoint_t *endpoint, const CAErrorInf
             {
                 OIC_LOG(INFO, TAG, "Now, finalize Direct-Pairing procedure.");
 
-                res = FinalizeDirectPairing(peer, resultCallback);
+                res = FinalizeDirectPairing(g_dp_proceed_ctx->userCtx, peer, resultCallback);
                 if(OC_STACK_OK != res)
                 {
                     OIC_LOG(ERROR, TAG, "Failed to finalize direct-pairing");
-                    resultCallback(peer, res);
+                    resultCallback(g_dp_proceed_ctx->userCtx, peer, res);
                 }
             }
             else if(CA_DTLS_AUTHENTICATION_FAILURE == info->result)
             {
                 OIC_LOG(INFO, TAG, "DirectPairingDTLSHandshakeCB - Authentication failed");
-                resultCallback(peer, OC_STACK_AUTHENTICATION_FAILURE);
+                resultCallback(g_dp_proceed_ctx->userCtx, peer, OC_STACK_AUTHENTICATION_FAILURE);
             }
 
-#ifdef __WITH_DTLS__
-            CARegisterDTLSHandshakeCallback(NULL);
-#endif // __WITH_DTLS__
+#if defined(__WITH_DTLS__) || defined(__WITH_TLS__)
+            CAregisterSslHandshakeCallback(NULL);
+#endif // __WITH_DTLS__ or __WITH_TLS__
             res = RemoveCredential(&peer->deviceID);
             if(OC_STACK_RESOURCE_DELETED != res)
             {
@@ -586,12 +618,12 @@ static OCStackApplicationResult DirectPairingHandler(void *ctx, OCDoHandle UNUSE
 
     if (clientResponse)
     {
-        if(OC_STACK_RESOURCE_CREATED == clientResponse->result)
+        if(OC_STACK_RESOURCE_CHANGED == clientResponse->result)
         {
             // result
             OIC_LOG(INFO, TAG, "DirectPairingHandler : success POST request to /oic/sec/dpairing");
 
-#ifdef __WITH_DTLS__
+#if defined(__WITH_DTLS__) || defined(__WITH_TLS__)
             // Add temporary psk
             res = AddTmpPskWithPIN(&dpairData->peer->deviceID,
                            SYMMETRIC_PAIR_WISE_KEY,
@@ -606,12 +638,12 @@ static OCStackApplicationResult DirectPairingHandler(void *ctx, OCDoHandle UNUSE
             caresult = CAEnableAnonECDHCipherSuite(false);
             VERIFY_SUCCESS(TAG, CA_STATUS_OK == caresult, ERROR);
 
-            caresult = CASelectCipherSuite(TLS_ECDHE_PSK_WITH_AES_128_CBC_SHA_256);
+            caresult = CASelectCipherSuite(TLS_ECDHE_PSK_WITH_AES_128_CBC_SHA_256, CA_ADAPTER_IP);
             VERIFY_SUCCESS(TAG, CA_STATUS_OK == caresult, ERROR);
 
             //Register proceeding peer info. & DTLS event handler to catch the dtls event while handshake
             g_dp_proceed_ctx = dpairData;
-            res = CARegisterDTLSHandshakeCallback(DirectPairingDTLSHandshakeCB);
+            res = CAregisterSslHandshakeCallback(DirectPairingDTLSHandshakeCB);
             VERIFY_SUCCESS(TAG, CA_STATUS_OK == caresult, ERROR);
 
             // initiate dtls
@@ -625,7 +657,7 @@ static OCStackApplicationResult DirectPairingHandler(void *ctx, OCDoHandle UNUSE
             caresult = CAInitiateHandshake(endpoint);
             OICFree(endpoint);
             VERIFY_SUCCESS(TAG, CA_STATUS_OK == caresult, ERROR);
-#endif // __WITH_DTLS__
+#endif // __WITH_DTLS__ or __WITH_TLS__
 
             res = OC_STACK_OK;
         }
@@ -640,9 +672,9 @@ static OCStackApplicationResult DirectPairingHandler(void *ctx, OCDoHandle UNUSE
         OIC_LOG(ERROR, TAG, "DirectPairingHandler received Null clientResponse");
     }
 
-#ifdef __WITH_DTLS__
+#if defined(__WITH_DTLS__) || defined(__WITH_TLS__)
 exit:
-#endif // __WITH_DTLS__
+#endif // __WITH_DTLS__ or __WITH_TLS__
 
     if (OC_STACK_OK != res)
     {
@@ -653,7 +685,7 @@ exit:
             g_dp_proceed_ctx = NULL;
         }
 
-        resultCallback(dpairData->peer, res);
+        resultCallback(dpairData->userCtx, dpairData->peer, res);
     }
     OIC_LOG_V(INFO, TAG, "OUT DirectPairingHandler.");
     return OC_STACK_DELETE_TRANSACTION;
@@ -668,8 +700,8 @@ exit:
  *
  * @return OC_STACK_OK on success otherwise error.
  */
-OCStackResult DPDirectPairing(OCDirectPairingDev_t* peer, OicSecPrm_t pmSel, char *pinNumber,
-                                                     OCDirectPairingResultCB resultCallback)
+OCStackResult DPDirectPairing(void *ctx, OCDirectPairingDev_t* peer, OicSecPrm_t pmSel,
+                                char *pinNumber, OCDirectPairingResultCB resultCallback)
 {
     if(NULL == peer || NULL == pinNumber)
     {
@@ -705,8 +737,8 @@ OCStackResult DPDirectPairing(OCDirectPairingDev_t* peer, OicSecPrm_t pmSel, cha
         OIC_LOG(ERROR, TAG, "Failed to DpairingToCBORPayload");
         return OC_STACK_NO_MEMORY;
     }
-    OIC_LOG(INFO, TAG, "DPARING CBOR data:");
-    OIC_LOG_BUFFER(INFO, TAG, secPayload->securityData, secPayload->payloadSize);
+    OIC_LOG(DEBUG, TAG, "DPARING CBOR data:");
+    OIC_LOG_BUFFER(DEBUG, TAG, secPayload->securityData, secPayload->payloadSize);
 
     char query[MAX_URI_LENGTH + MAX_QUERY_LENGTH] = {0};
     if(!DPGenerateQuery(false,
@@ -732,6 +764,7 @@ OCStackResult DPDirectPairing(OCDirectPairingDev_t* peer, OicSecPrm_t pmSel, cha
     dpairData->peer = peer;
     memcpy(dpairData->pin, pinNumber, DP_PIN_LENGTH);
     dpairData->resultCallback = resultCallback;
+    dpairData->userCtx = ctx;
 
     OCCallbackData cbData =  {.context=NULL, .cb=NULL, .cd=NULL};
     cbData.cb = DirectPairingHandler;
@@ -950,9 +983,6 @@ OCStackResult DPDeviceDiscovery(unsigned short waittime)
     }
 
     OCStackResult ret;
-    struct timespec startTime = {.tv_sec=0, .tv_nsec=0};
-    struct timespec currTime  = {.tv_sec=0, .tv_nsec=0};
-    struct timespec timeout;
 
     const char DP_DISCOVERY_QUERY[] = "/oic/sec/pconf";
 
@@ -973,12 +1003,16 @@ OCStackResult DPDeviceDiscovery(unsigned short waittime)
     }
 
     // wait..
-    timeout.tv_sec  = 0;
-    timeout.tv_nsec = 100000000L;
 
     int clock_res = -1;
+#if defined(_MSC_VER)
+    time_t startTime = NULL;
+    clock_res = (time(&startTime) == -1);
+#else
+    struct timespec startTime = {.tv_sec=0, .tv_nsec=0};
 #if defined(__ANDROID__) || _POSIX_TIMERS > 0
     clock_res = clock_gettime(CLOCK_MONOTONIC, &startTime);
+#endif
 #endif
     if (0 != clock_res)
     {
@@ -992,8 +1026,14 @@ OCStackResult DPDeviceDiscovery(unsigned short waittime)
 
     while (1)
     {
+#if defined(_MSC_VER)
+        time_t currTime = NULL;
+        clock_res = (time(&currTime) == -1);
+#else
+        struct timespec currTime  = {.tv_sec=0, .tv_nsec=0};
 #if defined(__ANDROID__) || _POSIX_TIMERS > 0
         clock_res = clock_gettime(CLOCK_MONOTONIC, &currTime);
+#endif
 #endif
         if (0 != clock_res)
         {
@@ -1001,18 +1041,24 @@ OCStackResult DPDeviceDiscovery(unsigned short waittime)
             ret = OC_STACK_ERROR;
             break;
         }
+#if defined(_MSC_VER)
+        long elapsed = currTime - startTime;
+#else
         long elapsed = (currTime.tv_sec - startTime.tv_sec);
+#endif
         if (elapsed > waittime)
         {
             break;
         }
         else
         {
+            struct timespec timeout = {.tv_sec=0, .tv_nsec=100000000L};
+            OCProcess();
             nanosleep(&timeout, NULL);
         }
     }
 
-    //Waiting for each response.
+    // Waiting for each response.
     ret = OCCancel(handle, OC_LOW_QOS, NULL, 0);
     if (OC_STACK_OK != ret)
     {
