@@ -20,12 +20,13 @@
 
 #include <string.h>
 #include <math.h>
+#include "platform_features.h"
 #include "pbkdf2.h"
 #include "hmac.h"
 #include "debug.h"
 #include "logger.h"
 
-#define TAG "PBDKF2"
+#define TAG "OIC_PBDKF2"
 #define XOR_BUF(in, out, bufSize)\
 do \
 {\
@@ -74,8 +75,8 @@ int DeriveCryptoKeyFromPassword(const unsigned char *passwd, size_t pLen,
                                 const size_t keyLen, uint8_t *derivedKey)
 {
     int res = 0;
-    uint8_t buf[DTLS_HMAC_DIGEST_SIZE];
-    uint8_t uBuf[DTLS_HMAC_DIGEST_SIZE];
+    uint8_t buf[DTLS_HMAC_DIGEST_SIZE] = {0,};
+    uint8_t uBuf[DTLS_HMAC_DIGEST_SIZE] = {0,};
 
     size_t nBlocks = 0;
     size_t nOctetInLastBlock = 0;
