@@ -18,18 +18,27 @@
 //
 //-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=
 
+#include "iotivity_config.h"
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 #include <signal.h>
+#ifdef HAVE_UNISTD_H
 #include <unistd.h>
+#endif
+#ifdef HAVE_WINDOWS_H
+#include <windows.h>
+#endif
 #include <ocstack.h>
 #include <iostream>
 #include <sstream>
+#include <getopt.h>
 #include "ocpayload.h"
 #include "payload_logging.h"
 #include "logger.h"
-const char *getResult(OCStackResult result);
+#include "common.h"
+#include "oic_string.h"
+
 std::string getQueryStrForGetPut();
 
 #define TAG ("occlient")
@@ -50,6 +59,8 @@ typedef enum
     TEST_UNKNOWN_RESOURCE_GET_DEFAULT,
     TEST_UNKNOWN_RESOURCE_GET_BATCH,
     TEST_UNKNOWN_RESOURCE_GET_LINK_LIST,
+    TEST_GET_EMPTY,
+    TEST_GET_NULL,
     MAX_TESTS
 } CLIENT_TEST;
 
@@ -83,6 +94,8 @@ testToTextMap queryInterface[] = {
         {"?if=oic.if.baseline", TEST_PUT_DEFAULT},
         {"?if=oic.if.b", TEST_PUT_BATCH},
         {"?if=oic.if.ll", TEST_PUT_LINK_LIST},
+        {"", TEST_GET_EMPTY},
+        {NULL, TEST_GET_NULL},
 };
 
 
@@ -153,6 +166,11 @@ void PrintUsage()
                  "unavailable resource using batch interface.");
     OIC_LOG(INFO, TAG, "Test Case 9 : Discover Resources && Initiate GET Request on an "\
                  "unavailable resource using link list interface.");
+    OIC_LOG(INFO, TAG, "Test Case 10 : Discover Resources && Initiate GET Request on an "\
+                 "empty resource.");
+    OIC_LOG(INFO, TAG, "Test Case 11 : Discover Resources && Initiate GET Request on an "\
+                 "NULL resource.");
+
 }
 
 OCStackApplicationResult putReqCB(void* ctx, OCDoHandle /*handle*/,
@@ -350,9 +368,9 @@ int InitDiscovery()
     OCStackResult ret;
     OCCallbackData cbData;
     /* Start a discovery query*/
-    char szQueryUri[64] = { 0 };
+    char szQueryUri[MAX_QUERY_LENGTH] = { 0 };
 
-    strcpy(szQueryUri, RESOURCE_DISCOVERY_QUERY);
+    OICStrcpy(szQueryUri, sizeof(RESOURCE_DISCOVERY_QUERY), RESOURCE_DISCOVERY_QUERY);
 
     cbData.cb = discoveryReqCB;
     cbData.context = (void*)DEFAULT_CONTEXT_VALUE;
@@ -439,4 +457,3 @@ std::string getQueryStrForGetPut()
 {
     return "/a/room";
 }
-
