@@ -68,6 +68,10 @@ namespace OC
         virtual OCStackResult setDefaultDeviceEntityHandler(EntityHandler entityHandler);
 
         virtual OCStackResult sendResponse(const std::shared_ptr<OCResourceResponse> pResponse);
+
+        virtual OCStackResult setPropertyValue(OCPayloadType type, const std::string& tag, const std::string& value);
+        virtual OCStackResult getPropertyValue(OCPayloadType type, const std::string& tag, std::string& value);
+
     private:
         void processFunc();
         std::thread m_processThread;
