@@ -43,12 +43,18 @@ namespace OIC
                 /**
                 * Constructor for ProtocolBridgeResource
                 */
-                ProtocolBridgeResource() { };
+                ProtocolBridgeResource()
+                {
+
+                };
 
                 /**
                 * Virtual destructor for ProtocolBridgeResource
                 */
-                virtual ~ProtocolBridgeResource() { };
+                virtual ~ProtocolBridgeResource()
+                {
+
+                };
 
                 /**
                 * Initialize attributes of the resource
@@ -65,9 +71,12 @@ namespace OIC
                 * The implementor of the function can decide weather to notify OIC clients
                 * about the changed state or not.
                 *
+                * @param queryParams Request parameters
+                *
                 * @return Value of all attributes
                 */
-                virtual RCSResourceAttributes handleGetAttributesRequest() = 0;
+                virtual RCSResourceAttributes handleGetAttributesRequest(const
+                        std::map< std::string, std::string > &queryParams) = 0;
 
                 /**
                 * This function should be implemented by the according bundle resource
@@ -82,10 +91,12 @@ namespace OIC
                 * about the changed state or not.
                 *
                 * @param attrs Attributes to set
+                * @param queryParams Request parameters
                 *
                 * @return void
                 */
-                virtual void handleSetAttributesRequest(const RCSResourceAttributes &attrs) = 0;
+                virtual void handleSetAttributesRequest(const RCSResourceAttributes &attrs,
+                                                        const std::map< std::string, std::string > &queryParams) = 0;
         };
     }
 }
