@@ -39,19 +39,23 @@ RemoteResourceUnit::~RemoteResourceUnit()
     {
         if(remoteObject->isCaching())
         {
-            try{
+            try
+            {
                 remoteObject->stopCaching();
             }
-            catch(std::exception &e){
+            catch(std::exception &e)
+            {
                 OIC_LOG_V(ERROR, CONTAINER_TAG, "%s", e.what());
             }
         }
         if(remoteObject->isMonitoring())
         {
-            try{
+            try
+            {
                 remoteObject->stopMonitoring();
             }
-            catch(std::exception &e){
+            catch(std::exception &e)
+            {
                 OIC_LOG_V(ERROR, CONTAINER_TAG, "%s", e.what());
             }
         }
@@ -68,19 +72,6 @@ RemoteResourceUnit::Ptr RemoteResourceUnit::createRemoteResourceInfo(
     return retRemoteResourceUnit;
 }
 
-RemoteResourceUnit::Ptr RemoteResourceUnit::createRemoteResourceInfoWithStateCB(
-    RCSRemoteResourceObject::Ptr ptr, UpdatedCBFromServer updatedCB,
-    RCSRemoteResourceObject::StateChangedCallback stateCB)
-{
-    RemoteResourceUnit::Ptr retRemoteResourceUnit = std::make_shared<RemoteResourceUnit>();
-    retRemoteResourceUnit->remoteObject = ptr;
-    retRemoteResourceUnit->pUpdatedCB = updatedCB;
-
-    retRemoteResourceUnit->pStateChangedCB = stateCB;
-
-    return retRemoteResourceUnit;
-}
-
 RemoteResourceUnit::Ptr RemoteResourceUnit::createRemoteResourceInfoWithCacheCB(
     RCSRemoteResourceObject::Ptr ptr, UpdatedCBFromServer updatedCB,
     RCSRemoteResourceObject::CacheUpdatedCallback cacheCB)
