@@ -29,7 +29,7 @@ struct CloseApp{};
 constexpr int RESOURCE_TEMP = 1;
 constexpr int RESOURCE_LIGHT = 2;
 
-constexpr int DEFALUT_SERVER = 1;
+constexpr int DEFAULT_SERVER = 1;
 constexpr int CUSTOM_SERVER = 2;
 
 constexpr int INCREASE = 1;
@@ -59,8 +59,14 @@ int processUserInput(int min, int max)
 
     if (!std::cin.fail())
     {
-        if(input == max + 1) throw CloseApp();
-        if(min <= input && input <= max) return input;
+        if (input == max + 1)
+        {
+            throw CloseApp();
+        }
+        if (min <= input && input <= max)
+        {
+            return input;
+        }
     }
 
     std::cin.clear();
@@ -89,7 +95,7 @@ void displayControlLightMenu()
 
 void printAttributes(const RCSResourceAttributes& attrs)
 {
-    for(const auto& attr : attrs)
+    for (const auto& attr : attrs)
     {
         std::cout << "\tkey : " << attr.key() << "\n\tvalue : "
                   << attr.value().toString() << std::endl;
@@ -144,7 +150,7 @@ void updateAttribute(const std::string& attrKey, int control)
         attrs[attrKey] = attrs[attrKey].get<int>() + diff;
     }
 
-    if(control == INCREASE)
+    if (control == INCREASE)
     {
         std::cout << attrKey << " increased." << std::endl;
     }
@@ -204,13 +210,13 @@ void runResourceTypeSelection(int resourceMode)
 void runResourceModeSelection()
 {
     std::cout << "========================================================          \n";
-    std::cout << DEFALUT_SERVER << ". Creation of Simple Resource Without Handlers  \n";
+    std::cout << DEFAULT_SERVER << ". Creation of Simple Resource Without Handlers  \n";
     std::cout << CUSTOM_SERVER << ". Creation of Resource With Set and Get Handlers \n";
     std::cout << CUSTOM_SERVER + 1 << ". Quit                                       \n";
     std::cout << "========================================================          \n";
 
     g_currentRun = std::bind(runResourceTypeSelection,
-            processUserInput(DEFALUT_SERVER, CUSTOM_SERVER));
+            processUserInput(DEFAULT_SERVER, CUSTOM_SERVER));
 }
 
 void runPresenceSelection()
@@ -237,17 +243,17 @@ int main(void)
 {
     g_currentRun = runPresenceSelection;
 
-    while(true)
+    while (true)
     {
         try
         {
             g_currentRun();
         }
-        catch(const std::exception& e)
+        catch (const std::exception& e)
         {
             std::cout << e.what() << std::endl;
         }
-        catch(const CloseApp&)
+        catch (const CloseApp&)
         {
             break;
         }
@@ -256,7 +262,7 @@ int main(void)
 
     g_resource.reset();
 
-    if(g_isPresenceStarted)
+    if (g_isPresenceStarted)
     {
         stopPresence();
     }
