@@ -40,9 +40,16 @@ class ResourceCacheManagerTest : public TestWithMock
         {
             TestWithMock::SetUp();
             cacheInstance = ResourceCacheManager::getInstance();
-            pResource = PrimitiveResource::Ptr(mocks.Mock< PrimitiveResource >(), [](PrimitiveResource *) {});
+            pResource = PrimitiveResource::Ptr(mocks.Mock< PrimitiveResource >(),
+                                               [](PrimitiveResource *)
+                                               {
+
+                                               });
             mocks.OnCall(pResource.get(), PrimitiveResource::isObservable).Return(false);
-            cb = ([](std::shared_ptr<PrimitiveResource >, const RCSResourceAttributes &)->OCStackResult {return OC_STACK_OK;});
+            cb = ([](std::shared_ptr<PrimitiveResource >, const RCSResourceAttributes &)->OCStackResult
+                    {
+                        return OC_STACK_OK;
+                    });
         }
 
         virtual void TearDown()
@@ -57,10 +64,11 @@ TEST_F(ResourceCacheManagerTest, requestResourceCache_resourceIsNULL)
 
     CacheCB func = cb;
     REPORT_FREQUENCY rf = REPORT_FREQUENCY::UPTODATE;
+    CACHE_METHOD cm = CACHE_METHOD::ITERATED_GET;
     long reportTime = 20l;
 
-    ASSERT_THROW(cacheInstance->requestResourceCache(NULL, func, rf, reportTime),
-                 ResourceCacheManager::InvalidParameterException);
+    ASSERT_THROW(cacheInstance->requestResourceCache(NULL, func, cm, rf, reportTime),
+                 RCSInvalidParameterException);
 }
 
 TEST_F(ResourceCacheManagerTest, requestResourceCache_cacheCBIsNULL)
@@ -68,10 +76,11 @@ TEST_F(ResourceCacheManagerTest, requestResourceCache_cacheCBIsNULL)
 
     CacheCB func = NULL;
     REPORT_FREQUENCY rf = REPORT_FREQUENCY::UPTODATE;
+    CACHE_METHOD cm = CACHE_METHOD::ITERATED_GET;
     long reportTime = 20l;
 
-    ASSERT_THROW(cacheInstance->requestResourceCache(pResource, func, rf, reportTime),
-                 ResourceCacheManager::InvalidParameterException);
+    ASSERT_THROW(cacheInstance->requestResourceCache(pResource, func, cm, rf, reportTime),
+                 RCSInvalidParameterException);
 }
 
 TEST_F(ResourceCacheManagerTest, requestResourceCache_reportTimeIsNULL)
@@ -84,8 +93,9 @@ TEST_F(ResourceCacheManagerTest, requestResourceCache_reportTimeIsNULL)
 
     CacheCB func = cb;
     REPORT_FREQUENCY rf = REPORT_FREQUENCY::UPTODATE;
+    CACHE_METHOD cm = CACHE_METHOD::ITERATED_GET;
 
-    id = cacheInstance->requestResourceCache(pResource, func, rf);
+    id = cacheInstance->requestResourceCache(pResource, func, cm, rf);
     cacheInstance->cancelResourceCache(id);
 
     ASSERT_NE(id, 0);
@@ -102,9 +112,10 @@ TEST_F(ResourceCacheManagerTest, requestResourceCache_normalCase)
 
     CacheCB func = cb;
     REPORT_FREQUENCY rf = REPORT_FREQUENCY::UPTODATE;
+    CACHE_METHOD cm = CACHE_METHOD::ITERATED_GET;
     long reportTime = 20l;
 
-    id = cacheInstance->requestResourceCache(pResource, func, rf, reportTime);
+    id = cacheInstance->requestResourceCache(pResource, func, cm, rf, reportTime);
     cacheInstance->cancelResourceCache(id);
 
     ASSERT_NE(id, 0);
@@ -114,7 +125,7 @@ TEST_F(ResourceCacheManagerTest, cancelResourceCache_cacheIDIsZero)
 {
 
     ASSERT_THROW(cacheInstance->cancelResourceCache(0),
-                 ResourceCacheManager::InvalidParameterException);
+                 RCSInvalidParameterException);
 }
 
 TEST_F(ResourceCacheManagerTest, cancelResourceCache_normalCase)
@@ -127,46 +138,10 @@ TEST_F(ResourceCacheManagerTest, cancelResourceCache_normalCase)
 
     CacheCB func = cb;
     REPORT_FREQUENCY rf = REPORT_FREQUENCY::UPTODATE;
+    CACHE_METHOD cm = CACHE_METHOD::ITERATED_GET;
     long reportTime = 20l;
 
-    id = cacheInstance->requestResourceCache(pResource, func, rf, reportTime);
-
-    cacheInstance->cancelResourceCache(id);
-}
-
-TEST_F(ResourceCacheManagerTest, updateResourceCachePrimitiveResource_resourceIsNULL)
-{
-
-    pResource = NULL;
-
-    ASSERT_THROW(cacheInstance->updateResourceCache(pResource),
-                 ResourceCacheManager::InvalidParameterException);
-}
-
-TEST_F(ResourceCacheManagerTest, updateResourceCachePrimitiveResource_cacheIsNULL)
-{
-
-    ASSERT_THROW(cacheInstance->updateResourceCache(pResource),
-                 ResourceCacheManager::InvalidParameterException);
-}
-
-TEST_F(ResourceCacheManagerTest, updateResourceCachePrimitiveResource_normalCase)
-{
-
-    mocks.OnCall(pResource.get(), PrimitiveResource::requestGet);
-    mocks.OnCall(pResource.get(), PrimitiveResource::isObservable).Return(true);
-    mocks.OnCall(pResource.get(), PrimitiveResource::requestObserve);
-    mocks.OnCall(pResource.get(), PrimitiveResource::getUri).Return("testUri");
-    mocks.OnCall(pResource.get(), PrimitiveResource::getHost).Return("testHost");
-    mocks.OnCall(pResource.get(), PrimitiveResource::cancelObserve);
-
-    CacheCB func = cb;
-    REPORT_FREQUENCY rf = REPORT_FREQUENCY::UPTODATE;
-    long reportTime = 20l;
-
-    id = cacheInstance->requestResourceCache(pResource, func, rf, reportTime);
-
-    cacheInstance->updateResourceCache(pResource);
+    id = cacheInstance->requestResourceCache(pResource, func, cm, rf, reportTime);
 
     cacheInstance->cancelResourceCache(id);
 }
@@ -175,14 +150,14 @@ TEST_F(ResourceCacheManagerTest, updateResourceCacheCacheID_cacheIDIsZero)
 {
 
     ASSERT_THROW(cacheInstance->updateResourceCache(0),
-                 ResourceCacheManager::InvalidParameterException);
+                 RCSInvalidParameterException);
 }
 
 TEST_F(ResourceCacheManagerTest, updateResourceCacheCacheID_cacheIsNULL)
 {
 
     ASSERT_THROW(cacheInstance->updateResourceCache(id),
-                 ResourceCacheManager::InvalidParameterException);
+                 RCSInvalidParameterException);
 }
 
 TEST_F(ResourceCacheManagerTest, updateResourceCacheCacheID_normalCase)
@@ -197,85 +172,33 @@ TEST_F(ResourceCacheManagerTest, updateResourceCacheCacheID_normalCase)
 
     CacheCB func = cb;
     REPORT_FREQUENCY rf = REPORT_FREQUENCY::UPTODATE;
+    CACHE_METHOD cm = CACHE_METHOD::ITERATED_GET;
     long reportTime = 20l;
 
-    id = cacheInstance->requestResourceCache(pResource, func, rf, reportTime);
+    id = cacheInstance->requestResourceCache(pResource, func, cm, rf, reportTime);
 
     cacheInstance->updateResourceCache(id);
 
     cacheInstance->cancelResourceCache(id);
 }
 
-TEST_F(ResourceCacheManagerTest, getCachedDataPrimitiveResource_resourceIsNULL)
-{
-
-    pResource = NULL;
-
-    ASSERT_THROW(cacheInstance->getCachedData(pResource),
-                 ResourceCacheManager::InvalidParameterException);
-}
-
-TEST_F(ResourceCacheManagerTest, getCachedDataPrimitiveResource_handlerIsNULL)
-{
-
-    ASSERT_THROW(cacheInstance->getCachedData(pResource),
-                 ResourceCacheManager::InvalidParameterException);
-}
-
 TEST_F(ResourceCacheManagerTest, getCachedDataCachID_resourceIsNULL)
 {
 
-    ASSERT_THROW(cacheInstance->getCachedData(0), ResourceCacheManager::InvalidParameterException);
+    ASSERT_THROW(cacheInstance->getCachedData(0), RCSInvalidParameterException);
 }
 
 TEST_F(ResourceCacheManagerTest, getCachedDataCachID_handlerIsNULL)
 {
 
-    ASSERT_THROW(cacheInstance->getCachedData(id), ResourceCacheManager::InvalidParameterException);
-}
-
-TEST_F(ResourceCacheManagerTest, getResourceCacheStatePrimitiveResource_resourceIsNULL)
-{
-
-    pResource = NULL;
-
-    ASSERT_THROW(cacheInstance->getResourceCacheState(pResource),
-                 ResourceCacheManager::InvalidParameterException);
-}
-
-TEST_F(ResourceCacheManagerTest, getResourceCacheStatePrimitiveResource_handlerIsNULL)
-{
-
-    ASSERT_EQ(cacheInstance->getResourceCacheState(pResource), CACHE_STATE::NONE);
-}
-
-TEST_F(ResourceCacheManagerTest, getResourceCacheStatePrimitiveResource_normalCase)
-{
-
-    mocks.OnCall(pResource.get(), PrimitiveResource::requestGet);
-    mocks.OnCall(pResource.get(), PrimitiveResource::isObservable).Return(true);
-    mocks.OnCall(pResource.get(), PrimitiveResource::requestObserve);
-    mocks.OnCall(pResource.get(), PrimitiveResource::getUri).Return("testUri");
-    mocks.OnCall(pResource.get(), PrimitiveResource::getHost).Return("testHost");
-    mocks.OnCall(pResource.get(), PrimitiveResource::cancelObserve);
-
-    CacheCB func = cb;
-    REPORT_FREQUENCY rf = REPORT_FREQUENCY::UPTODATE;
-    long reportTime = 20l;
-
-    id = cacheInstance->requestResourceCache(pResource, func, rf, reportTime);
-    CACHE_STATE state = cacheInstance->getResourceCacheState(pResource);
-
-    cacheInstance->cancelResourceCache(id);
-
-    ASSERT_EQ(state, CACHE_STATE::READY_YET);
+    ASSERT_THROW(cacheInstance->getCachedData(id), RCSInvalidParameterException);
 }
 
 TEST_F(ResourceCacheManagerTest, getResourceCacheStateCacheID_cacheIDIsZero)
 {
 
     ASSERT_THROW(cacheInstance->getResourceCacheState(0),
-                 ResourceCacheManager::InvalidParameterException);
+                 RCSInvalidParameterException);
 }
 
 TEST_F(ResourceCacheManagerTest, getResourceCacheStateCacheID_handlerIsNULL)
@@ -297,9 +220,10 @@ TEST_F(ResourceCacheManagerTest, getResourceCacheStateCacheID_normalCase)
 
     CacheCB func = cb;
     REPORT_FREQUENCY rf = REPORT_FREQUENCY::UPTODATE;
+    CACHE_METHOD cm = CACHE_METHOD::ITERATED_GET;
     long reportTime = 20l;
 
-    id = cacheInstance->requestResourceCache(pResource, func, rf, reportTime);
+    id = cacheInstance->requestResourceCache(pResource, func, cm, rf, reportTime);
     CACHE_STATE state = cacheInstance->getResourceCacheState(id);
 
     cacheInstance->cancelResourceCache(id);
