@@ -364,10 +364,8 @@ class ResourceObjectHandlingRequestTest: public ResourceObjectTest
 public:
     EntityHandler handler;
 
-    static constexpr OCRequestHandle fakeRequestHandle =
-            reinterpret_cast<OCRequestHandle>(0x1234);
-    static constexpr OCResourceHandle fakeResourceHandle =
-            reinterpret_cast<OCResourceHandle>(0x4321);
+    static const OCRequestHandle fakeRequestHandle;
+    static const OCResourceHandle fakeResourceHandle;
 
 public:
     OCResourceRequest::Ptr createRequest(OCMethod method = OC_REST_GET, OCRepresentation ocRep =
@@ -415,6 +413,10 @@ protected:
     }
 };
 
+const OCRequestHandle ResourceObjectHandlingRequestTest::fakeRequestHandle = 0x1234;
+const OCResourceHandle  ResourceObjectHandlingRequestTest::fakeResourceHandle =
+    reinterpret_cast<OCResourceHandle>(0x4321);
+
 TEST_F(ResourceObjectHandlingRequestTest, CallSendResponseWhenReceiveRequest)
 {
     mocks.ExpectCallFunc(OCPlatform::sendResponse).Return(OC_STACK_OK);
@@ -698,7 +700,7 @@ TEST_F(ResourceObjectInterfaceHandlerTest, ThrowIfDefaultInterfaceIsInvalid)
 
 TEST_F(ResourceObjectInterfaceHandlerTest, SettingDefaultInterfaceEqualsGetDefaultInterface)
 {
-    initServer({SENSOR_INTERFACE}, BASELINE_INTERFACE);
+    initServer({ SENSOR_INTERFACE }, BASELINE_INTERFACE);
 
     EXPECT_EQ(BASELINE_INTERFACE, server->getDefaultInterface());
 }
