@@ -40,7 +40,23 @@ constexpr char RESOURCE_URI[]{ "/a/TemperatureSensor" };
 constexpr char RESOURCE_TYPE[]{ "resource.type" };
 constexpr char SECOND_RESOURCETYPE[]{ "resource.type.second" };
 
-void onResourceDiscovered(RCSRemoteResourceObject::Ptr) {}
+#ifdef SECURED
+const char * SVR_DB_FILE_NAME = "./oic_svr_db_re_client.dat";
+//OCPersistent Storage Handlers
+static FILE* client_open(const char * /*path*/, const char *mode)
+{
+    std::string file_name = SVR_DB_FILE_NAME;
+#ifndef LOCAL_RUNNING
+    file_name = "./service/resource-encapsulation/unittests/oic_svr_db_re_client.dat";
+#endif
+    return fopen(file_name.c_str(), mode);
+}
+#endif
+
+void onResourceDiscovered(RCSRemoteResourceObject::Ptr)
+{
+
+}
 
 class ScopedTask
 {
@@ -52,7 +68,10 @@ public:
 
     ~ScopedTask()
     {
-        if (m_task) m_task->cancel();
+        if (m_task)
+        {
+            m_task->cancel();
+        }
     }
 
     RCSDiscoveryManager::DiscoveryTask* operator->()
@@ -69,6 +88,14 @@ private:
 
 TEST(DiscoveryManagerTest, ThrowIfDiscoverWithEmptyCallback)
 {
+#ifdef SECURED
+    OCPersistentStorage gps {client_open, fread, fwrite, fclose, unlink };
+    OC::PlatformConfig cfg
+    { OC::ServiceType::InProc, OC::ModeType::Both, "0.0.0.0", 0,
+            OC::QualityOfService::LowQos, &gps };
+    OC::OCPlatform::Configure(cfg);
+#endif
+
     EXPECT_THROW(RCSDiscoveryManager::getInstance()->discoverResource(RCSAddress::multicast(), { }),
             RCSInvalidParameterException);
 }
@@ -126,7 +153,8 @@ TEST(DiscoveryManagerTest, TaskCanBeCanceled)
     ASSERT_TRUE(aTaskToBeCanceled->isCanceled());
 }
 
-TEST(DiscoveryManagerTest, CallbackWouldNotBeCalledForSameRemoteResource) {
+TEST(DiscoveryManagerTest, CallbackWouldNotBeCalledForSameRemoteResource)
+{
     FindCallback callback;
 
     MockRepository mocks;
