@@ -30,6 +30,7 @@ typedef enum
     SIMULATOR_RESOURCE_CREATED,
     SIMULATOR_RESOURCE_DELETED,
     SIMULATOR_CONTINUE,
+    SIMULATOR_RESOURCE_CHANGED,
     SIMULATOR_INVALID_URI = 20,
     SIMULATOR_INVALID_QUERY,
     SIMULATOR_INVALID_IP,
@@ -57,6 +58,11 @@ typedef enum
     SIMULATOR_INVALID_DEVICE_INFO,
     SIMULATOR_INVALID_JSON,
     SIMULATOR_UNAUTHORIZED_REQ,
+    SIMULATOR_TOO_LARGE_REQ,
+    SIMULATOR_PDM_IS_NOT_INITIALIZED,
+    SIMULATOR_DUPLICATE_UUID,
+    SIMULATOR_INCONSISTENT_DB,
+    SIMULATOR_AUTHENTICATION_FAILURE,
 #ifdef WITH_PRESENCE
     SIMULATOR_PRESENCE_STOPPED = 128,
     SIMULATOR_PRESENCE_TIMEOUT,
