@@ -18,25 +18,35 @@
  *
  ******************************************************************/
 
-#ifndef REQUEST_MODEL_BUILDER_H_
-#define REQUEST_MODEL_BUILDER_H_
+#ifndef SIMULATOR_REQUEST_MODEL_BUILDER_H_
+#define SIMULATOR_REQUEST_MODEL_BUILDER_H_
 
 #include "request_model.h"
 #include "response_model.h"
 
+namespace RAML
+{
+    class Raml;
+    class RamlResource;
+    class Action;
+    class Response;
+    class RequestResponseBody;
+}
+
 class RequestModelBuilder
 {
     public:
-        RequestModelBuilder(std::shared_ptr<RAML::Raml> &raml);
-        std::map<RequestType, RequestModelSP> build(const std::string &uri);
+        std::unordered_map<std::string, RequestModelSP> build(
+            const std::shared_ptr<RAML::Raml> &raml, const std::string &uri);
+        std::unordered_map<std::string, RequestModelSP> build(
+            const std::shared_ptr<RAML::RamlResource> &resource);
 
     private:
-        RequestModelSP createRequestModel(const RAML::ActionPtr &action);
-        ResponseModelSP createResponseModel(int code, const RAML::ResponsePtr &response);
-        SimulatorResourceModelSP createRepSchema(const RAML::RequestResponseBodyPtr &rep);
-        RequestType getRequestType(RAML::ActionType actionType);
-
-        std::shared_ptr<RAML::Raml> m_raml;
+        RequestModelSP createRequestModel(const std::shared_ptr<RAML::Action> &action);
+        ResponseModelSP createResponseModel(int code,
+                                            const std::shared_ptr<RAML::Response> &response);
+        std::shared_ptr<SimulatorResourceModelSchema> createRepSchema(
+            const std::shared_ptr<RAML::RequestResponseBody> &rep);
 };
 
 #endif
